@@ -457,9 +457,10 @@ void NativeDebugger::handleVariables(QJsonObject *response, const QJsonObject &a
         QV4::Heap::InternalClass *ic = callContext->internalClass();
         QV4::ScopedValue v(scope);
         for (uint i = 0; i < ic->size; ++i) {
-            QString name = ic->keyAt(i);
-            v = callContext->d()->locals[i];
-            collector.collect(&output, QString(), name, v);
+            QV4::ScopedValue stringOrSymbol(scope, ic->keyAt(i));
+            QV4::ScopedString propName(scope, stringOrSymbol->toString(scope.engine));
+            v = callContext->getProperty(propName);
+            collector.collect(&output, QString(), propName->toQString(), v);
         }
     }
 
