@@ -2,24 +2,27 @@
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qv4object_p.h"
-#include "qv4objectproto_p.h"
-#include "qv4stringobject_p.h"
-#include "qv4argumentsobject_p.h"
+
+#include <private/qv4argumentsobject_p.h>
+#include <private/qv4identifiertable_p.h>
+#include <private/qv4jscall_p.h>
+#include <private/qv4lookup_p.h>
+#include <private/qv4memberdata_p.h>
 #include <private/qv4mm_p.h>
-#include "qv4lookup_p.h"
-#include "qv4scopedvalue_p.h"
-#include "qv4memberdata_p.h"
-#include "qv4objectiterator_p.h"
-#include "qv4identifierhash_p.h"
-#include "qv4string_p.h"
-#include "qv4identifiertable_p.h"
-#include "qv4jscall_p.h"
-#include "qv4symbol_p.h"
-#include "qv4proxy_p.h"
+#include <private/qv4proxy_p.h>
+#include <private/qv4scopedvalue_p.h>
+#include <private/qv4stackframe_p.h>
+#include <private/qv4stringobject_p.h>
+#include <private/qv4symbol_p.h>
+
+#include <QtCore/qloggingcategory.h>
 
 #include <stdint.h>
 
 using namespace QV4;
+using namespace Qt::Literals::StringLiterals;
+
+Q_LOGGING_CATEGORY(lcJavaScriptGlobals, "qt.qml.js.globals")
 
 DEFINE_OBJECT_VTABLE(Object);
 
@@ -39,8 +42,13 @@ void Object::setInternalClass(Heap::InternalClass *ic)
         // Pick the members of the old IC that are still valid in the new IC.
         // Order them by index in memberData (or inline data).
         Scoped<MemberData> newMembers(scope, MemberData::allocate(scope.engine, ic->size));
-        for (uint i = 0; i < ic->size; ++i)
-            newMembers->set(scope.engine, i, get(ic->nameMap.at(i)));
+        for (uint i = 0; i < ic->size; ++i) {
+            // Note that some members might have been deleted. The key may be invalid.
+            const PropertyKey key = ic->nameMap.at(i);
+            // fromReturnedValue is safe, we directly write it through the WriteBarrier
+            newMembers->set(scope.engine, i,
+                            QV4::Value::fromReturnedValue(key.isValid() ? get(key) : Encode::undefined()));
+        }
 
         p->internalClass.set(scope.engine, ic);
         const uint nInline = p->vtable()->nInlineProperties;
@@ -68,7 +76,9 @@ void Object::setInternalClass(Heap::InternalClass *ic)
         }
     }
 
-    if (ic->isUsedAsProto())
+    // Before the engine is done initializing, we cannot have any lookups.
+    // Therefore, there is no point in updating the proto IDs.
+    if (ic->engine->isInitialized && ic->isUsedAsProto())
         ic->updateProtoUsage(p);
 
 }
@@ -175,16 +185,16 @@ void Object::defineAccessorProperty(StringOrSymbol *name, VTable::Call getter, V
     QV4::Scope scope(v4);
     ScopedProperty p(scope);
     QString n = name->toQString();
-    if (n.at(0) == QLatin1Char('@'))
-        n = QChar::fromLatin1('[') + QStringView{n}.mid(1) + QChar::fromLatin1(']');
+    if (!n.isEmpty() && n.at(0) == '@'_L1)
+        n = '['_L1 + QStringView{n}.mid(1) + ']'_L1;
     if (getter) {
-        ScopedString getName(scope, v4->newString(QString::fromLatin1("get ") + n));
+        ScopedString getName(scope, v4->newString("get "_L1 + n));
         p->setGetter(ScopedFunctionObject(scope, FunctionObject::createBuiltinFunction(v4, getName, getter, 0)));
     } else {
         p->setGetter(nullptr);
     }
     if (setter) {
-        ScopedString setName(scope, v4->newString(QString::fromLatin1("set ") + n));
+        ScopedString setName(scope, v4->newString("set "_L1 + n));
         p->setSetter(ScopedFunctionObject(scope, FunctionObject::createBuiltinFunction(v4, setName, setter, 0)));
     } else {
         p->setSetter(nullptr);
@@ -734,8 +744,17 @@ ReturnedValue Object::virtualInstanceOf(const Object *typeObject, const Value &v
 
 ReturnedValue Object::virtualResolveLookupGetter(const Object *object, ExecutionEngine *engine, Lookup *lookup)
 {
+    // Otherwise we cannot trust the protoIds
+    Q_ASSERT(engine->isInitialized);
+
     Heap::Object *obj = object->d();
     PropertyKey name = engine->identifierTable->asPropertyKey(engine->currentStackFrame->v4Function->compilationUnit->runtimeStrings[lookup->nameIndex]);
+    if (object->as<QV4::ProxyObject>()) {
+        // proxies invalidate assumptions that we normally maek in lookups
+        // so we always need to use the fallback path
+        lookup->getter = Lookup::getterFallback;
+        return lookup->getter(lookup, engine, *object);
+    }
     if (name.isArrayIndex()) {
         lookup->indexedLookup.index = name.asArrayIndex();
         lookup->getter = Lookup::getterIndexed;
@@ -757,7 +776,7 @@ ReturnedValue Object::virtualResolveLookupGetter(const Object *object, Execution
         } else {
             lookup->getter = Lookup::getterAccessor;
         }
-        lookup->objectLookup.ic = obj->internalClass;
+        lookup->objectLookup.ic.set(engine, obj->internalClass.get());
         lookup->objectLookup.offset = index.index;
         return lookup->getter(lookup, engine, *object);
     }
@@ -769,6 +788,9 @@ ReturnedValue Object::virtualResolveLookupGetter(const Object *object, Execution
 
 bool Object::virtualResolveLookupSetter(Object *object, ExecutionEngine *engine, Lookup *lookup, const Value &value)
 {
+    // Otherwise we cannot trust the protoIds
+    Q_ASSERT(engine->isInitialized);
+
     Scope scope(engine);
     ScopedString name(scope, scope.engine->currentStackFrame->v4Function->compilationUnit->runtimeStrings[lookup->nameIndex]);
 
@@ -781,7 +803,7 @@ bool Object::virtualResolveLookupSetter(Object *object, ExecutionEngine *engine,
             lookup->setter = Lookup::arrayLengthSetter;
             return lookup->setter(lookup, engine, *object, value);
         } else if (idx.attrs.isData() && idx.attrs.isWritable()) {
-            lookup->objectLookup.ic = object->internalClass();
+            lookup->objectLookup.ic.set(engine, object->internalClass());
             lookup->objectLookup.index = idx.index;
             const auto nInline = object->d()->vtable()->nInlineProperties;
             if (idx.index < nInline) {
@@ -815,12 +837,37 @@ bool Object::virtualResolveLookupSetter(Object *object, ExecutionEngine *engine,
         lookup->setter = Lookup::setterFallback;
         return false;
     }
-    lookup->insertionLookup.newClass = object->internalClass();
+    lookup->insertionLookup.newClass.set(engine, object->internalClass());
     lookup->insertionLookup.offset = idx.index;
     lookup->setter = Lookup::setterInsert;
     return true;
 }
 
+/*!
+ * \internal
+ *
+ * This method is modeled after \l{QMetaObject::metacall}. It takes a JavaScript
+ * \a object and performs \a call on it, using \a index as identifier for the
+ * property/method/etc to be used and \a a as arguments. Like
+ * \l{QMetaObject::metacall} this method is overly generic in order to reduce the
+ * API surface. On a plain Object it does nothing and returns 0. Specific
+ * objects can override it and do some meaningful work. If the metacall succeeds
+ * they should return a non-0 value. Otherwise they should return 0.
+ *
+ * Most prominently, \l QMetaObject::ReadProperty and \l QMetaObject::WriteProperty
+ * calls can be used to manipulate properties of QObjects and Q_GADGETs wrapped
+ * by QV4::QObjectWrapper, QV4::QQmlTypeWrapper and QV4::QQmlValueTypeWrapper.
+ * They can also be used to manipulate elements in QV4::Sequence.
+ */
+int Object::virtualMetacall(Object *object, QMetaObject::Call call, int index, void **a)
+{
+    Q_UNUSED(object);
+    Q_UNUSED(call);
+    Q_UNUSED(index);
+    Q_UNUSED(a);
+    return 0;
+}
+
 ReturnedValue Object::checkedInstanceOf(ExecutionEngine *engine, const FunctionObject *f, const Value &var)
 {
     Scope scope(engine);
@@ -938,12 +985,29 @@ bool Object::virtualDefineOwnProperty(Managed *m, PropertyKey id, const Property
         return o->internalDefineOwnProperty(scope.engine, index, nullptr, p, attrs);
     }
 
-    auto memberIndex = o->internalClass()->find(id);
+    Scoped<InternalClass> ic(scope, o->internalClass());
+    auto memberIndex = ic->d()->find(id);
 
     if (!memberIndex.isValid()) {
         if (!o->isExtensible())
             return false;
 
+        // If the IC is locked, you're not allowed to shadow any unconfigurable properties.
+        if (ic->d()->isLocked()) {
+            while (Heap::Object *prototype = ic->d()->prototype) {
+                ic = prototype->internalClass;
+                const auto entry = ic->d()->find(id);
+                if (entry.isValid()) {
+                    if (entry.attributes.isConfigurable())
+                        break;
+                    qCWarning(lcJavaScriptGlobals).noquote()
+                            << QStringLiteral("You cannot shadow the locked property "
+                                              "'%1' in QML.").arg(id.toQString());
+                    return false;
+                }
+            }
+        }
+
         Scoped<StringOrSymbol> name(scope, id.asStringOrSymbol());
         ScopedProperty pd(scope);
         pd->copy(p, attrs);
@@ -977,11 +1041,12 @@ bool Object::virtualSetPrototypeOf(Managed *m, const Object *proto)
 {
     Q_ASSERT(m->isObject());
     Object *o = static_cast<Object *>(m);
-    Heap::Object *current = o->internalClass()->prototype;
+    Heap::InternalClass *ic = o->internalClass();
+    Heap::Object *current = ic->prototype;
     Heap::Object *protod = proto ? proto->d() : nullptr;
     if (current == protod)
         return true;
-    if (!o->internalClass()->isExtensible())
+    if (!ic->isExtensible() || ic->isLocked())
         return false;
     Heap::Object *p = protod;
     while (p) {
@@ -991,7 +1056,7 @@ bool Object::virtualSetPrototypeOf(Managed *m, const Object *proto)
             break;
         p = p->prototype();
     }
-    o->setInternalClass(o->internalClass()->changePrototype(protod));
+    o->setInternalClass(ic->changePrototype(protod));
     return true;
 }
 
@@ -1127,6 +1192,7 @@ QStringList ArrayObject::toQStringList() const
     ScopedValue v(scope);
 
     uint length = getLength();
+    result.reserve(length);
     for (uint i = 0; i < length; ++i) {
         v = const_cast<ArrayObject *>(this)->get(i);
         result.append(v->toQStringNoThrow());
