@@ -78,6 +78,12 @@ QQmlType QQmlTypeData::qmlType(const QString &inlineComponentName) const
 
 bool QQmlTypeData::tryLoadFromDiskCache()
 {
+    if (!m_backupSourceCode.isCacheable())
+        return false;
+
+    if (auto unit = QQmlMetaType::obtainCompilationUnit(url()))
+        return loadFromDiskCache(unit);
+
     if (!readCacheFile())
         return false;
 
@@ -442,7 +448,7 @@ void QQmlTypeData::done()
             ? m_document.data()->isSingleton()
             : (m_compiledData->unitData()->flags & QV4::CompiledData::Unit::IsSingleton);
         m_qmlType = QQmlMetaType::findCompositeType(
-                finalUrl(), m_compiledData, isSingleton
+                url(), m_compiledData, isSingleton
                         ? QQmlMetaType::Singleton
                         : QQmlMetaType::NonSingleton);
         m_typeClassName = QByteArray(m_qmlType.typeId().name()).chopped(1);
@@ -676,11 +682,9 @@ void QQmlTypeData::initializeFromCachedUnit(const QQmlPrivate::CachedQmlUnit *un
         return;
     }
 
-    m_document.reset(new QmlIR::Document(isDebugging()));
+    m_document.reset(new QmlIR::Document(urlString(), finalUrlString(), isDebugging()));
     QQmlIRLoader loader(unit->qmlData, m_document.data());
     loader.load();
-    m_document->jsModule.fileName = urlString();
-    m_document->jsModule.finalUrl = finalUrlString();
     m_document->javaScriptCompilationUnit
             = QQmlRefPointer<QV4::CompiledData::CompilationUnit>(
                 new QV4::CompiledData::CompilationUnit(unit->qmlData, unit->aotCompiledFunctions),
@@ -690,7 +694,7 @@ void QQmlTypeData::initializeFromCachedUnit(const QQmlPrivate::CachedQmlUnit *un
 
 bool QQmlTypeData::loadFromSource()
 {
-    m_document.reset(new QmlIR::Document(isDebugging()));
+    m_document.reset(new QmlIR::Document(urlString(), finalUrlString(), isDebugging()));
     m_document->jsModule.sourceTimeStamp = m_backupSourceCode.sourceTimeStamp();
     QQmlEngine *qmlEngine = typeLoader()->engine();
     QmlIR::IRBuilder compiler(qmlEngine->handle()->illegalNames());
@@ -721,11 +725,9 @@ bool QQmlTypeData::loadFromSource()
 
 void QQmlTypeData::restoreIR(const QQmlRefPointer<QV4::CompiledData::CompilationUnit> &unit)
 {
-    m_document.reset(new QmlIR::Document(isDebugging()));
+    m_document.reset(new QmlIR::Document(urlString(), finalUrlString(), isDebugging()));
     QQmlIRLoader loader(unit->unitData(), m_document.data());
     loader.load();
-    m_document->jsModule.fileName = urlString();
-    m_document->jsModule.finalUrl = finalUrlString();
     m_document->javaScriptCompilationUnit = unit;
     continueLoadFromIR();
 }
