@@ -4,6 +4,7 @@
 #include "quicktest_p.h"
 #include "quicktestresult_p.h"
 #include <QtTest/qtestsystem.h>
+#include <QtTest/private/qtestcrashhandler_p.h>
 #include "qtestoptions_p.h"
 #include <QtQml/qqml.h>
 #include <QtQml/qqmlengine.h>
@@ -31,10 +32,16 @@
 #include <QtGui/qtextdocument.h>
 #include <stdio.h>
 #include <QtGui/QGuiApplication>
+#include <QtGui/private/qguiapplication_p.h>
+#include <QtGui/qpa/qplatformintegration.h>
 #include <QtCore/QTranslator>
 #include <QtTest/QSignalSpy>
 #include <QtQml/QQmlFileSelector>
 
+#ifdef Q_OS_ANDROID
+#include <QtCore/QStandardPaths>
+#endif
+
 #include <private/qqmlcomponent_p.h>
 #include <private/qv4resolvedtypereference_p.h>
 
@@ -92,6 +99,9 @@ bool QQuickTest::qIsPolishScheduled(const QQuickItem *item)
             QVERIFY(QQuickTest::qWaitForPolish(window));
     \endcode
 
+    The QML equivalent of this function is
+    \l [QML]{TestCase::}{isPolishScheduled()}.
+
     \sa QQuickItem::polish(), QQuickItem::updatePolish(),
         QQuickTest::qWaitForPolish()
 */
@@ -151,6 +161,9 @@ bool QQuickTest::qWaitForPolish(const QQuickItem *item, int timeout)
     Returns \c true if \c qIsPolishScheduled(item) returns false for all items
     within \a timeout milliseconds, otherwise returns \c false.
 
+    The QML equivalent of this function is
+    \l [QML]{TestCase::}{waitForPolish()}.
+
     \sa QQuickItem::polish(), QQuickItem::updatePolish(),
         QQuickTest::qIsPolishScheduled()
 */
@@ -217,25 +230,49 @@ static void handleCompileErrors(
     results.stopLogging();
 }
 
-bool qWaitForSignal(QObject *obj, const char* signal, int timeout = 5000)
-{
-    QSignalSpy spy(obj, signal);
-    QElapsedTimer timer;
-    timer.start();
+class SimpleReceiver : public QObject {
+    Q_OBJECT
+public:
+    bool signalReceived = false;
+public slots:
+    void slotFun() { signalReceived = true; }
+};
 
-    while (!spy.size()) {
-        int remaining = timeout - int(timer.elapsed());
-        if (remaining <= 0)
-            break;
-        QCoreApplication::processEvents(QEventLoop::AllEvents, remaining);
-        QCoreApplication::sendPostedEvents(nullptr, QEvent::DeferredDelete);
-        QTest::qSleep(10);
+bool qWaitForSignal(QObject *obj, const char* signal, int timeout)
+{
+    if (!obj || !signal) {
+        qWarning("qWaitForSignal: invalid arguments");
+        return false;
+    }
+    if (((signal[0] - '0') & 0x03) != QSIGNAL_CODE) {
+        qWarning("qWaitForSignal: not a valid signal, use the SIGNAL macro");
+        return false;
     }
 
-    return spy.size();
+    int sig = obj->metaObject()->indexOfSignal(signal + 1);
+    if (sig == -1) {
+        const QByteArray ba = QMetaObject::normalizedSignature(signal + 1);
+        sig = obj->metaObject()->indexOfSignal(ba.constData());
+        if (sig == -1) {
+            qWarning("qWaitForSignal: no such signal %s::%s", obj->metaObject()->className(),
+                     signal);
+            return false;
+        }
+    }
+
+    SimpleReceiver receiver;
+    static int slot = receiver.metaObject()->indexOfSlot("slotFun()");
+    if (!QMetaObject::connect(obj, sig, &receiver, slot)) {
+        qWarning("qWaitForSignal: failed to connect to signal %s::%s",
+                 obj->metaObject()->className(), signal);
+        return false;
+    }
+
+    return QTest::qWaitFor([&]() { return receiver.signalReceived; }, timeout);
 }
 
-void maybeInvokeSetupMethod(QObject *setupObject, const char *member, QGenericArgument val0 = QGenericArgument(nullptr))
+template <typename... Args>
+void maybeInvokeSetupMethod(QObject *setupObject, const char *member, Args &&... args)
 {
     // It's OK if it doesn't exist: since we have more than one callback that
     // can be called, it makes sense if the user only implements one of them.
@@ -246,7 +283,7 @@ void maybeInvokeSetupMethod(QObject *setupObject, const char *member, QGenericAr
     const int methodIndex = setupMetaObject->indexOfMethod(member);
     if (methodIndex != -1) {
         const QMetaMethod method = setupMetaObject->method(methodIndex);
-        method.invoke(setupObject, val0);
+        method.invoke(setupObject, std::forward<Args>(args)...);
     }
 }
 
@@ -257,7 +294,7 @@ class TestCaseCollector
 public:
     typedef QList<QString> TestCaseList;
 
-    TestCaseCollector(const QFileInfo &fileInfo, QQmlEngine *engine)
+    TestCaseCollector(const QFileInfo &fileInfo, QQmlEngine *engine) : m_engine(engine)
     {
         QString path = fileInfo.absoluteFilePath();
         if (path.startsWith(QLatin1String(":/")))
@@ -269,7 +306,8 @@ public:
         if (component.isReady()) {
             QQmlRefPointer<QV4::ExecutableCompilationUnit> rootCompilationUnit
                     = QQmlComponentPrivate::get(&component)->compilationUnit;
-            TestCaseEnumerationResult result = enumerateTestCases(rootCompilationUnit.data());
+            TestCaseEnumerationResult result = enumerateTestCases(
+                    rootCompilationUnit->baseCompilationUnit().data());
             m_testCases = result.testCases + result.finalizedPartialTestCases();
             m_errors += result.errors;
         }
@@ -281,6 +319,7 @@ public:
 private:
     TestCaseList m_testCases;
     QList<QQmlError> m_errors;
+    QQmlEngine *m_engine = nullptr;
 
     struct TestCaseEnumerationResult
     {
@@ -309,8 +348,8 @@ private:
     };
 
     TestCaseEnumerationResult enumerateTestCases(
-            const QQmlRefPointer<QV4::ExecutableCompilationUnit> &compilationUnit,
-            const Object *object = nullptr)
+            const QQmlRefPointer<QV4::CompiledData::CompilationUnit> &compilationUnit,
+            const QV4::CompiledData::Object *object = nullptr)
     {
         QQmlType testCaseType;
         for (quint32 i = 0, count = compilationUnit->importCount(); i < count; ++i) {
@@ -323,7 +362,8 @@ private:
             if (!typeQualifier.isEmpty())
                 testCaseTypeName = typeQualifier % QLatin1Char('.') % testCaseTypeName;
 
-            testCaseType = compilationUnit->typeNameCache->query(testCaseTypeName).type;
+            testCaseType = compilationUnit->typeNameCache->query(
+                    testCaseTypeName, QQmlTypeLoader::get(m_engine)).type;
             if (testCaseType.isValid())
                 break;
         }
@@ -332,11 +372,11 @@ private:
 
         if (!object) // Start at root of compilation unit if not enumerating a specific child
             object = compilationUnit->objectAt(0);
-        if (object->hasFlag(Object::IsInlineComponentRoot))
+        if (object->hasFlag(QV4::CompiledData::Object::IsInlineComponentRoot))
             return result;
 
-        if (const auto superTypeUnit = compilationUnit->resolvedTypes.value(
-                    object->inheritedTypeNameIndex)->compilationUnit()) {
+        if (const auto superTypeUnit = compilationUnit->resolvedType(object->inheritedTypeNameIndex)
+                                               ->compilationUnit()) {
             // We have a non-C++ super type, which could indicate we're a subtype of a TestCase
             if (testCaseType.isValid() && superTypeUnit->url() == testCaseType.sourceUrl())
                 result.isTestCase = true;
@@ -379,7 +419,7 @@ private:
 
         for (auto binding = object->bindingsBegin(); binding != object->bindingsEnd(); ++binding) {
             if (binding->type() == QV4::CompiledData::Binding::Type_Object) {
-                const Object *child = compilationUnit->objectAt(binding->value.objectIndex);
+                const QV4::CompiledData::Object *child = compilationUnit->objectAt(binding->value.objectIndex);
                 result << enumerateTestCases(compilationUnit, child);
             }
         }
@@ -393,12 +433,24 @@ int quick_test_main(int argc, char **argv, const char *name, const char *sourceD
     return quick_test_main_with_setup(argc, argv, name, sourceDir, nullptr);
 }
 
+#ifdef Q_OS_ANDROID
+static QFile androidExitCodeFile()
+{
+    const QString testHome = QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
+    return QFile(testHome + "/qtest_last_exit_code");
+}
+#endif
+
 int quick_test_main_with_setup(int argc, char **argv, const char *name, const char *sourceDir, QObject *setup)
 {
     QScopedPointer<QCoreApplication> app;
     if (!QCoreApplication::instance())
         app.reset(new QGuiApplication(argc, argv));
 
+#ifdef Q_OS_ANDROID
+    androidExitCodeFile().remove();
+#endif
+
     if (setup)
         maybeInvokeSetupMethod(setup, "applicationAvailable()");
 
@@ -534,6 +586,11 @@ int quick_test_main_with_setup(int argc, char **argv, const char *name, const ch
         return 1;
     }
 
+    std::optional<QTest::CrashHandler::FatalSignalHandler> handler;
+    QTest::CrashHandler::prepareStackTrace();
+    if (!QTest::Internal::noCrashHandler)
+        handler.emplace();
+
     qputenv("QT_QTESTLIB_RUNNING", "1");
 
     QSet<QString> commandLineTestFunctions(QTest::testFunctions.cbegin(), QTest::testFunctions.cend());
@@ -609,39 +666,35 @@ int quick_test_main_with_setup(int argc, char **argv, const char *name, const ch
             handleCompileErrors(fi, view.errors(), view.engine(), &view);
             continue;
         }
-        if (!QTestRootObject::instance()->hasQuit) {
-            // If the test already quit, then it was performed
-            // synchronously during setSource().  Otherwise it is
-            // an asynchronous test and we need to show the window
-            // and wait for the first frame to be rendered
-            // and then wait for quit indication.
-            view.setFramePosition(QPoint(50, 50));
-            if (view.size().isEmpty()) { // Avoid hangs with empty windows.
-                view.resize(200, 200);
-            }
-            view.show();
-            if (!QTest::qWaitForWindowExposed(&view)) {
-                qWarning().nospace()
-                    << "Test '" << QDir::toNativeSeparators(path) << "' window not exposed after show().";
-            }
+
+        view.setFramePosition(QPoint(50, 50));
+        if (view.size().isEmpty()) { // Avoid hangs with empty windows.
+            view.resize(200, 200);
+        }
+        view.show();
+        if (!QTest::qWaitForWindowExposed(&view)) {
+            qWarning().nospace()
+                << "Test '" << QDir::toNativeSeparators(path) << "' window not exposed after show().";
+        }
+        if (QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowActivation)) {
             view.requestActivate();
             if (!QTest::qWaitForWindowActive(&view)) {
                 qWarning().nospace()
                     << "Test '" << QDir::toNativeSeparators(path) << "' window not active after requestActivate().";
             }
-            if (view.isExposed()) {
-                // Defer property update until event loop has started
-                QTimer::singleShot(0, []() {
-                    QTestRootObject::instance()->setWindowShown(true);
-                });
-            } else {
-                qWarning().nospace()
-                    << "Test '" << QDir::toNativeSeparators(path) << "' window was never exposed! "
-                    << "If the test case was expecting windowShown, it will hang.";
-            }
-            if (!QTestRootObject::instance()->hasQuit && QTestRootObject::instance()->hasTestCase())
-                eventLoop.exec();
         }
+        if (view.isExposed()) {
+            // Defer property update until event loop has started
+            QTimer::singleShot(0, []() {
+                QTestRootObject::instance()->setWindowShown(true);
+            });
+        } else {
+            qWarning().nospace()
+                << "Test '" << QDir::toNativeSeparators(path) << "' window was never exposed! "
+                << "If the test case was expecting windowShown, it will hang.";
+        }
+        if (!QTestRootObject::instance()->hasQuit && QTestRootObject::instance()->hasTestCase())
+            eventLoop.exec();
     }
 
     if (setup)
@@ -659,10 +712,23 @@ int quick_test_main_with_setup(int argc, char **argv, const char *name, const ch
         return commandLineTestFunctions.size();
     }
 
+    const int exitCode = QuickTestResult::exitCode();
+
+#ifdef Q_OS_ANDROID
+    QFile exitCodeFile = androidExitCodeFile();
+    if (exitCodeFile.open(QIODevice::WriteOnly)) {
+        exitCodeFile.write(qPrintable(QString::number(exitCode)));
+    } else {
+        qWarning("Failed to open %s for writing test exit code: %s",
+                 qPrintable(exitCodeFile.fileName()), qPrintable(exitCodeFile.errorString()));
+    }
+#endif
+
     // Return the number of failures as the exit code.
-    return QuickTestResult::exitCode();
+    return exitCode;
 }
 
 QT_END_NAMESPACE
 
 #include "moc_quicktest_p.cpp"
+#include "quicktest.moc"
