@@ -10,6 +10,7 @@
 QT_BEGIN_NAMESPACE
 
 Q_LOGGING_CATEGORY(lcItemViewDelegateLifecycle, "qt.quick.itemview.lifecycle")
+Q_LOGGING_CATEGORY(lcCount, "qt.quick.itemview.count")
 
 // Default cacheBuffer for all views.
 #ifndef QML_VIEW_DEFAULTCACHEBUFFER
@@ -160,8 +161,11 @@ void QQuickItemView::setModel(const QVariant &m)
         disconnect(d->model, SIGNAL(createdItem(int,QObject*)), this, SLOT(createdItem(int,QObject*)));
         disconnect(d->model, SIGNAL(destroyingItem(QObject*)), this, SLOT(destroyingItem(QObject*)));
         if (QQmlDelegateModel *delegateModel = qobject_cast<QQmlDelegateModel*>(d->model)) {
-            disconnect(delegateModel, SIGNAL(itemPooled(int, QObject *)), this, SLOT(onItemPooled(int, QObject *)));
-            disconnect(delegateModel, SIGNAL(itemReused(int, QObject *)), this, SLOT(onItemReused(int, QObject *)));
+            disconnect(delegateModel, SIGNAL(itemPooled(int,QObject*)), this, SLOT(onItemPooled(int,QObject*)));
+            disconnect(delegateModel, SIGNAL(itemReused(int,QObject*)), this, SLOT(onItemReused(int,QObject*)));
+            QObjectPrivate::disconnect(
+                    delegateModel, &QQmlDelegateModel::delegateChanged,
+                    d, &QQuickItemViewPrivate::applyDelegateChange);
         }
     }
 
@@ -199,8 +203,8 @@ void QQuickItemView::setModel(const QVariant &m)
         connect(d->model, SIGNAL(initItem(int,QObject*)), this, SLOT(initItem(int,QObject*)));
         connect(d->model, SIGNAL(destroyingItem(QObject*)), this, SLOT(destroyingItem(QObject*)));
         if (QQmlDelegateModel *delegateModel = qobject_cast<QQmlDelegateModel*>(d->model)) {
-            connect(delegateModel, SIGNAL(itemPooled(int, QObject *)), this, SLOT(onItemPooled(int, QObject *)));
-            connect(delegateModel, SIGNAL(itemReused(int, QObject *)), this, SLOT(onItemReused(int, QObject *)));
+            connect(delegateModel, SIGNAL(itemPooled(int,QObject*)), this, SLOT(onItemPooled(int,QObject*)));
+            connect(delegateModel, SIGNAL(itemReused(int,QObject*)), this, SLOT(onItemReused(int,QObject*)));
         }
         if (isComponentComplete()) {
             d->updateSectionCriteria();
@@ -211,17 +215,19 @@ void QQuickItemView::setModel(const QVariant &m)
             setCurrentIndex(d->model->count() > 0 ? 0 : -1);
             d->updateViewport();
 
+#if QT_CONFIG(quick_viewtransitions)
             if (d->transitioner && d->transitioner->populateTransition) {
                 d->transitioner->setPopulateTransitionEnabled(true);
                 d->forceLayoutPolish();
             }
+#endif
         }
 
         connect(d->model, SIGNAL(modelUpdated(QQmlChangeSet,bool)),
                 this, SLOT(modelUpdated(QQmlChangeSet,bool)));
         if (QQmlDelegateModel *dataModel = qobject_cast<QQmlDelegateModel*>(d->model))
             QObjectPrivate::connect(dataModel, &QQmlDelegateModel::delegateChanged, d, &QQuickItemViewPrivate::applyDelegateChange);
-        emit countChanged();
+        d->emitCountChanged();
     }
     emit modelChanged();
     d->moveReason = QQuickItemViewPrivate::Other;
@@ -253,7 +259,7 @@ void QQuickItemView::setDelegate(QQmlComponent *delegate)
         int oldCount = dataModel->count();
         dataModel->setDelegate(delegate);
         if (oldCount != dataModel->count())
-            emit countChanged();
+            d->emitCountChanged();
     }
     emit delegateChanged();
     d->delegateValidated = false;
@@ -684,6 +690,7 @@ void QQuickItemView::setReuseItems(bool reuse)
     emit reuseItemsChanged();
 }
 
+#if QT_CONFIG(quick_viewtransitions)
 QQuickTransition *QQuickItemView::populateTransition() const
 {
     Q_D(const QQuickItemView);
@@ -811,6 +818,7 @@ void QQuickItemView::setDisplacedTransition(QQuickTransition *transition)
         emit displacedTransitionChanged();
     }
 }
+#endif
 
 void QQuickItemViewPrivate::positionViewAtIndex(int index, int mode)
 {
@@ -1082,8 +1090,7 @@ qreal QQuickItemViewPrivate::calculatedMaxExtent() const
 void QQuickItemViewPrivate::applyDelegateChange()
 {
     releaseVisibleItems(QQmlDelegateModel::NotReusable);
-    releaseItem(currentItem, QQmlDelegateModel::NotReusable);
-    currentItem = nullptr;
+    releaseCurrentItem(QQmlDelegateModel::NotReusable);
     updateSectionCriteria();
     refill();
     moveReason = QQuickItemViewPrivate::SetIndex;
@@ -1122,6 +1129,14 @@ void QQuickItemViewPrivate::showVisibleItems() const
     }
 }
 
+// Simplifies debugging of count.
+void QQuickItemViewPrivate::emitCountChanged()
+{
+    Q_Q(QQuickItemView);
+    qCDebug(lcCount).nospace() << "about to emit countChanged for " << q << "; count changed to " << q->count();
+    emit q->countChanged();
+}
+
 void QQuickItemViewPrivate::itemGeometryChanged(QQuickItem *item, QQuickGeometryChange change,
                                                 const QRectF &oldGeometry)
 {
@@ -1148,11 +1163,13 @@ void QQuickItemViewPrivate::itemGeometryChanged(QQuickItem *item, QQuickGeometry
         // don't allow item movement transitions to trigger a re-layout and
         // start new transitions
         bool prevInLayout = inLayout;
+#if QT_CONFIG(quick_viewtransitions)
         if (!inLayout) {
             FxViewItem *actualItem = transitioner ? visibleItem(currentIndex) : nullptr;
             if (actualItem && actualItem->transitionRunning())
                 inLayout = true;
         }
+#endif
         updateHighlight();
         inLayout = prevInLayout;
     }
@@ -1165,17 +1182,20 @@ void QQuickItemView::destroyRemoved()
 {
     Q_D(QQuickItemView);
 
+#if QT_CONFIG(quick_viewtransitions)
     bool hasRemoveTransition = false;
     bool hasRemoveTransitionAsTarget = false;
     if (d->transitioner) {
         hasRemoveTransition = d->transitioner->canTransition(QQuickItemViewTransitioner::RemoveTransition, false);
         hasRemoveTransitionAsTarget = d->transitioner->canTransition(QQuickItemViewTransitioner::RemoveTransition, true);
     }
+#endif
 
     for (QList<FxViewItem*>::Iterator it = d->visibleItems.begin();
             it != d->visibleItems.end();) {
         FxViewItem *item = *it;
         if (item->index == -1 && (!item->attached || item->attached->delayRemove() == false)) {
+#if QT_CONFIG(quick_viewtransitions)
             if (hasRemoveTransitionAsTarget) {
                 // don't remove from visibleItems until next layout()
                 d->runDelayedRemoveTransition = true;
@@ -1184,9 +1204,12 @@ void QQuickItemView::destroyRemoved()
             } else {
                 if (hasRemoveTransition)
                     d->runDelayedRemoveTransition = true;
+#endif
                 d->releaseItem(item, d->reusableFlag);
                 it = d->visibleItems.erase(it);
+#if QT_CONFIG(quick_viewtransitions)
             }
+#endif
         } else {
             ++it;
         }
@@ -1201,8 +1224,10 @@ void QQuickItemView::modelUpdated(const QQmlChangeSet &changeSet, bool reset)
     Q_D(QQuickItemView);
     if (reset) {
         cancelFlick();
+#if QT_CONFIG(quick_viewtransitions)
         if (d->transitioner)
             d->transitioner->setPopulateTransitionEnabled(true);
+#endif
         d->moveReason = QQuickItemViewPrivate::SetIndex;
         d->regenerate();
         if (d->highlight && d->currentItem) {
@@ -1211,9 +1236,11 @@ void QQuickItemView::modelUpdated(const QQmlChangeSet &changeSet, bool reset)
             d->updateTrackedItem();
         }
         d->moveReason = QQuickItemViewPrivate::Other;
-        emit countChanged();
+        d->emitCountChanged();
+#if QT_CONFIG(quick_viewtransitions)
         if (d->transitioner && d->transitioner->populateTransition)
             d->forceLayoutPolish();
+#endif
     } else {
         if (d->inLayout) {
             d->bufferedChanges.prepare(d->currentIndex, d->itemCount);
@@ -1251,7 +1278,9 @@ void QQuickItemView::trackedPositionChanged()
         return;
     }
 
-    if (d->moveReason == QQuickItemViewPrivate::SetIndex) {
+    const bool needMoveToTrackHighlight = d->autoHighlight || d->highlightRange != NoHighlightRange;
+
+    if (d->moveReason == QQuickItemViewPrivate::SetIndex && needMoveToTrackHighlight) {
         qreal trackedPos = d->trackedItem->position();
         qreal trackedSize = d->trackedItem->size();
         qreal viewPos = d->isContentFlowReversed() ? -d->position()-d->size() : d->position();
@@ -1449,8 +1478,10 @@ void QQuickItemView::componentComplete()
     d->updateFooter();
     d->updateViewport();
     d->setPosition(d->contentStartOffset());
+#if QT_CONFIG(quick_viewtransitions)
     if (d->transitioner)
         d->transitioner->setPopulateTransitionEnabled(true);
+#endif
 
     if (d->isValid()) {
         d->refill();
@@ -1468,7 +1499,7 @@ void QQuickItemView::componentComplete()
         d->fixupPosition();
     }
     if (d->model && d->model->count())
-        emit countChanged();
+        d->emitCountChanged();
 }
 
 
@@ -1478,7 +1509,6 @@ QQuickItemViewPrivate::QQuickItemViewPrivate()
     , buffer(QML_VIEW_DEFAULTCACHEBUFFER), bufferMode(BufferBefore | BufferAfter)
     , displayMarginBeginning(0), displayMarginEnd(0)
     , layoutDirection(Qt::LeftToRight), verticalLayoutDirection(QQuickItemView::TopToBottom)
-    , moveReason(Other)
     , visibleIndex(0)
     , currentIndex(-1), currentItem(nullptr)
     , trackedItem(nullptr), requestedIndex(-1)
@@ -1487,7 +1517,9 @@ QQuickItemViewPrivate::QQuickItemViewPrivate()
     , highlightRangeStart(0), highlightRangeEnd(0)
     , highlightMoveDuration(150)
     , headerComponent(nullptr), header(nullptr), footerComponent(nullptr), footer(nullptr)
+#if QT_CONFIG(quick_viewtransitions)
     , transitioner(nullptr)
+#endif
     , minExtent(0), maxExtent(0)
     , ownModel(false), wrap(false)
     , keyNavigationEnabled(true)
@@ -1495,7 +1527,10 @@ QQuickItemViewPrivate::QQuickItemViewPrivate()
     , inLayout(false), inViewportMoved(false), forceLayout(false), currentIndexCleared(false)
     , haveHighlightRange(false), autoHighlight(true), highlightRangeStartValid(false), highlightRangeEndValid(false)
     , fillCacheBuffer(false), inRequest(false)
-    , runDelayedRemoveTransition(false), delegateValidated(false), isClearing(false)
+#if QT_CONFIG(quick_viewtransitions)
+    , runDelayedRemoveTransition(false)
+#endif
+    , delegateValidated(false), isClearing(false)
 {
     bufferPause.addAnimationChangeListener(this, QAbstractAnimationJob::Completion);
     bufferPause.setLoopCount(1);
@@ -1504,9 +1539,11 @@ QQuickItemViewPrivate::QQuickItemViewPrivate()
 
 QQuickItemViewPrivate::~QQuickItemViewPrivate()
 {
+#if QT_CONFIG(quick_viewtransitions)
     if (transitioner)
         transitioner->setChangeListener(nullptr);
     delete transitioner;
+#endif
 }
 
 bool QQuickItemViewPrivate::isValid() const
@@ -1628,8 +1665,7 @@ void QQuickItemViewPrivate::updateCurrent(int modelIndex)
         if (currentItem) {
             if (currentItem->attached)
                 currentItem->attached->setIsCurrentItem(false);
-            releaseItem(currentItem, reusableFlag);
-            currentItem = nullptr;
+            releaseCurrentItem(reusableFlag);
             currentIndex = modelIndex;
             emit q->currentIndexChanged();
             emit q->currentItemChanged();
@@ -1682,16 +1718,17 @@ void QQuickItemViewPrivate::clear(bool onDestruction)
     releaseVisibleItems(QQmlInstanceModel::NotReusable);
     visibleIndex = 0;
 
+#if QT_CONFIG(quick_viewtransitions)
     for (FxViewItem *item : std::as_const(releasePendingTransition)) {
         item->releaseAfterTransition = false;
         releaseItem(item, QQmlInstanceModel::NotReusable);
     }
     releasePendingTransition.clear();
+#endif
 
-    auto oldCurrentItem = currentItem;
-    releaseItem(currentItem, QQmlDelegateModel::NotReusable);
-    currentItem = nullptr;
-    if (oldCurrentItem)
+    const bool hadCurrentItem = currentItem != nullptr;
+    releaseCurrentItem(QQmlDelegateModel::NotReusable);
+    if (hadCurrentItem)
         emit q->currentItemChanged();
     createHighlight(onDestruction);
     trackedItem = nullptr;
@@ -1736,6 +1773,8 @@ void QQuickItemViewPrivate::refill(qreal from, qreal to)
     Q_Q(QQuickItemView);
     if (!model || !model->isValid() || !q->isComponentComplete())
         return;
+    if (q->size().isNull() && visibleItems.isEmpty())
+        return;
     if (!model->count()) {
         updateHeader();
         updateFooter();
@@ -1759,7 +1798,6 @@ void QQuickItemViewPrivate::refill(qreal from, qreal to)
         qreal fillTo = to;
 
         bool added = addVisibleItems(fillFrom, fillTo, bufferFrom, bufferTo, false);
-        bool removed = removeNonVisibleItems(bufferFrom, bufferTo);
 
         if (requestedIndex == -1 && buffer && bufferMode != NoBuffer) {
             if (added) {
@@ -1775,6 +1813,8 @@ void QQuickItemViewPrivate::refill(qreal from, qreal to)
             }
         }
 
+        bool removed = removeNonVisibleItems(bufferFrom, bufferTo);
+
         if (added || removed) {
             markExtentsDirty();
             updateBeginningEnd();
@@ -1785,7 +1825,7 @@ void QQuickItemViewPrivate::refill(qreal from, qreal to)
         }
 
         if (prevCount != itemCount)
-            emit q->countChanged();
+            emitCountChanged();
     } while (currentChanges.hasPendingChanges() || bufferedChanges.hasPendingChanges());
     storeFirstVisibleItemPosition();
 }
@@ -1832,16 +1872,32 @@ void QQuickItemViewPrivate::layout()
     // viewBounds contains bounds before any add/remove/move operation to the view
     QRectF viewBounds(q->contentX(),  q->contentY(), q->width(), q->height());
 
-    if (!isValid() && !visibleItems.size()) {
+    // We use isNull for the size check, because isEmpty returns true
+    // if either dimension is negative, but apparently we support negative-sized
+    // views (see tst_QQuickListView::resizeView).
+    if ((!isValid() && !visibleItems.size()) || q->size().isNull()) {
+        if (q->size().isNull() && hasPendingChanges()) {
+            // count() refers to the number of items in the model, not in the view
+            // (which is why we don't emit for the !visibleItems.size() case).
+            // If there are pending model changes, emit countChanged in order to
+            // support the use case of QTBUG-129165, where visible is bound to count > 0
+            // and the ListView is in a layout with Layout.preferredHeight bound to
+            // contentHeight. This ensures that a hidden ListView will become visible.
+            emitCountChanged();
+        }
+
         clear();
         setPosition(contentStartOffset());
         updateViewport();
+#if QT_CONFIG(quick_viewtransitions)
         if (transitioner)
             transitioner->setPopulateTransitionEnabled(false);
+#endif
         inLayout = false;
         return;
     }
 
+#if QT_CONFIG(quick_viewtransitions)
     if (runDelayedRemoveTransition && transitioner
             && transitioner->canTransition(QQuickItemViewTransitioner::RemoveTransition, false)) {
         // assume that any items moving now are moving due to the remove - if they schedule
@@ -1849,6 +1905,7 @@ void QQuickItemViewPrivate::layout()
         for (int i=0; i<visibleItems.size(); i++)
             visibleItems[i]->transitionNextReposition(transitioner, QQuickItemViewTransitioner::RemoveTransition, false);
     }
+#endif
 
     ChangeResult insertionPosChanges;
     ChangeResult removalPosChanges;
@@ -1862,6 +1919,7 @@ void QQuickItemViewPrivate::layout()
     }
     forceLayout = false;
 
+#if QT_CONFIG(quick_viewtransitions)
     if (transitioner && transitioner->canTransition(QQuickItemViewTransitioner::PopulateTransition, true)) {
         // Give the view one more chance to refill itself,
         // in case its size is changed such that more delegates become visible after component completed
@@ -1871,12 +1929,15 @@ void QQuickItemViewPrivate::layout()
                 item->transitionNextReposition(transitioner, QQuickItemViewTransitioner::PopulateTransition, true);
         }
     }
+#endif
 
     updateSections();
     layoutVisibleItems();
     storeFirstVisibleItemPosition();
 
+#if QT_CONFIG(quick_viewtransitions)
     int lastIndexInView = findLastIndexInView();
+#endif
     refill();
     markExtentsDirty();
     updateHighlight();
@@ -1891,6 +1952,7 @@ void QQuickItemViewPrivate::layout()
     updateViewport();
     updateUnrequestedPositions();
 
+#if QT_CONFIG(quick_viewtransitions)
     if (transitioner) {
         // items added in the last refill() may need to be transitioned in - e.g. a remove
         // causes items to slide up into view
@@ -1931,8 +1993,14 @@ void QQuickItemViewPrivate::layout()
         transitioner->setPopulateTransitionEnabled(false);
         transitioner->resetTargetLists();
     }
+#endif
 
+    if (!currentItem)
+        updateCurrent(currentIndex);
+
+#if QT_CONFIG(quick_viewtransitions)
     runDelayedRemoveTransition = false;
+#endif
     inLayout = false;
 }
 
@@ -1988,6 +2056,7 @@ bool QQuickItemViewPrivate::applyModelChanges(ChangeResult *totalInsertionResult
                 removalResult.countChangeBeforeVisible += (correctedFirstVisibleIndex  - r.index);
         }
     }
+#if QT_CONFIG(quick_viewtransitions)
     if (runDelayedRemoveTransition) {
         QQmlChangeSet::Change removal;
         for (QList<FxViewItem*>::Iterator it = visibleItems.begin(); it != visibleItems.end();) {
@@ -2001,6 +2070,7 @@ bool QQuickItemViewPrivate::applyModelChanges(ChangeResult *totalInsertionResult
             }
         }
     }
+#endif
     *totalRemovalResult += removalResult;
     if (!removals.isEmpty()) {
         updateVisibleIndex();
@@ -2039,6 +2109,7 @@ bool QQuickItemViewPrivate::applyModelChanges(ChangeResult *totalInsertionResult
             item->attached->emitAdd();
     }
 
+#if QT_CONFIG(quick_viewtransitions)
     // for each item that was moved directly into the view as a result of a move(),
     // find the index it was moved from in order to set its initial position, so that we
     // can transition it from this "original" position to its new position in the view
@@ -2054,13 +2125,16 @@ bool QQuickItemViewPrivate::applyModelChanges(ChangeResult *totalInsertionResult
             }
         }
     }
+#endif
 
     // reposition visibleItems.first() correctly so that the content y doesn't jump
     if (removedCount != prevVisibleItemsCount)
         repositionFirstItem(prevVisibleItemsFirst, prevVisibleItemsFirstPos, prevFirstItemInView, &insertionResult, &removalResult);
 
     // Whatever removed/moved items remain are no longer visible items.
+#if QT_CONFIG(quick_viewtransitions)
     prepareRemoveTransitions(&currentChanges.removedItems);
+#endif
     for (auto it = currentChanges.removedItems.begin();
          it != currentChanges.removedItems.end(); ++it) {
         releaseItem(it.value(), reusableFlag);
@@ -2071,10 +2145,9 @@ bool QQuickItemViewPrivate::applyModelChanges(ChangeResult *totalInsertionResult
         if (currentChanges.currentRemoved && currentItem) {
             if (currentItem->item && currentItem->attached)
                 currentItem->attached->setIsCurrentItem(false);
-            auto oldCurrentItem = currentItem;
-            releaseItem(currentItem, reusableFlag);
-            currentItem = nullptr;
-            if (oldCurrentItem)
+            const bool hadCurrentItem = currentItem != nullptr;
+            releaseCurrentItem(reusableFlag);
+            if (hadCurrentItem)
                 emit q->currentItemChanged();
         }
         if (!currentIndexCleared)
@@ -2087,7 +2160,7 @@ bool QQuickItemViewPrivate::applyModelChanges(ChangeResult *totalInsertionResult
 
     updateSections();
     if (prevItemCount != itemCount)
-        emit q->countChanged();
+        emitCountChanged();
     if (!visibleAffected && viewportChanged)
         updateViewport();
 
@@ -2117,10 +2190,12 @@ bool QQuickItemViewPrivate::applyRemovalChange(const QQmlChangeSet::Change &remo
         } else if (item->index >= removal.index + removal.count) {
             // after removed items
             item->index -= removal.count;
+#if QT_CONFIG(quick_viewtransitions)
             if (removal.isMove())
                 item->transitionNextReposition(transitioner, QQuickItemViewTransitioner::MoveTransition, false);
             else
                 item->transitionNextReposition(transitioner, QQuickItemViewTransitioner::RemoveTransition, false);
+#endif
             ++it;
         } else {
             // removed item
@@ -2154,7 +2229,9 @@ void QQuickItemViewPrivate::removeItem(FxViewItem *item, const QQmlChangeSet::Ch
     }
     if (removal.isMove()) {
         currentChanges.removedItems.replace(removal.moveKey(item->index), item);
+#if QT_CONFIG(quick_viewtransitions)
         item->transitionNextReposition(transitioner, QQuickItemViewTransitioner::MoveTransition, true);
+#endif
     } else {
         // track item so it is released later
         currentChanges.removedItems.insert(QQmlChangeSet::MoveKey(), item);
@@ -2208,6 +2285,7 @@ void QQuickItemViewPrivate::repositionFirstItem(FxViewItem *prevVisibleItemsFirs
     }
 }
 
+#if QT_CONFIG(quick_viewtransitions)
 void QQuickItemViewPrivate::createTransitioner()
 {
     if (!transitioner) {
@@ -2283,6 +2361,7 @@ void QQuickItemViewPrivate::viewItemTransitionFinished(QQuickItemViewTransitiona
         }
     }
 }
+#endif
 
 /*
   This may return 0 if the item is being created asynchronously.
@@ -2296,6 +2375,7 @@ FxViewItem *QQuickItemViewPrivate::createItem(int modelIndex, QQmlIncubator::Inc
     if (requestedIndex == modelIndex && incubationMode == QQmlIncubator::Asynchronous)
         return nullptr;
 
+#if QT_CONFIG(quick_viewtransitions)
     for (int i=0; i<releasePendingTransition.size(); i++) {
         if (releasePendingTransition.at(i)->index == modelIndex
                 && !releasePendingTransition.at(i)->isPendingRemoval()) {
@@ -2303,6 +2383,7 @@ FxViewItem *QQuickItemViewPrivate::createItem(int modelIndex, QQmlIncubator::Inc
             return releasePendingTransition.takeAt(i);
         }
     }
+#endif
 
     inRequest = true;
 
@@ -2421,7 +2502,9 @@ bool QQuickItemViewPrivate::releaseItem(FxViewItem *item, QQmlInstanceModel::Reu
                 // One case where this can happen is moving an item out of one ObjectModel and into another.
                 QQuickItemPrivate::get(item->item)->setCulled(true);
             }
-            if (!isClearing)
+            // If deleteLater was called, the item isn't long for this world and so we shouldn't store references to it.
+            // This can happen when a Repeater is used to populate items in SwipeView's ListView contentItem.
+            if (!isClearing && !QObjectPrivate::get(item->item)->deleteLaterCalled)
                 unrequestedItems.insert(item->item, model->indexOf(item->item, q));
         } else if (flags & QQmlInstanceModel::Destroyed) {
             item->item->setParentItem(nullptr);
@@ -2433,9 +2516,15 @@ bool QQuickItemViewPrivate::releaseItem(FxViewItem *item, QQmlInstanceModel::Reu
     return flags != QQmlInstanceModel::Referenced;
 }
 
-QQuickItem *QQuickItemViewPrivate::createHighlightItem() const
+QQuickItem *QQuickItemViewPrivate::createHighlightItem()
 {
-    return createComponentItem(highlightComponent, 0.0, true);
+    QQuickItem *item = nullptr;
+    if (!inRequest) {
+        inRequest = true;
+        item = createComponentItem(highlightComponent, 0.0, true);
+        inRequest = false;
+    }
+    return item;
 }
 
 QQuickItem *QQuickItemViewPrivate::createComponentItem(QQmlComponent *component, qreal zValue, bool createDefault) const
@@ -2444,17 +2533,14 @@ QQuickItem *QQuickItemViewPrivate::createComponentItem(QQmlComponent *component,
 
     QQuickItem *item = nullptr;
     if (component) {
-        QQmlContext *creationContext = component->creationContext();
-        QQmlContext *context = new QQmlContext(
-                creationContext ? creationContext : qmlContext(q));
-        QObject *nobj = component->beginCreate(context);
-        if (nobj) {
-            QQml_setParent_noEvent(context, nobj);
+        QQmlContext *context = component->creationContext();
+        if (!context)
+            context = qmlContext(q);
+
+        if (QObject *nobj = component->beginCreate(context)) {
             item = qobject_cast<QQuickItem *>(nobj);
             if (!item)
                 delete nobj;
-        } else {
-            delete context;
         }
     } else if (createDefault) {
         item = new QQuickItem;
