@@ -21,7 +21,7 @@ Q_LOGGING_CATEGORY(lcPA, "qt.quick.pincharea")
 
 /*!
     \qmltype PinchEvent
-    \instantiates QQuickPinchEvent
+    \nativetype QQuickPinchEvent
     \inqmlmodule QtQuick
     \ingroup qtquick-input-events
     \brief For specifying information about a pinch event.
@@ -43,9 +43,9 @@ Q_LOGGING_CATEGORY(lcPA, "qt.quick.pincharea")
 */
 
 /*!
-    \qmlproperty QPointF QtQuick::PinchEvent::center
-    \qmlproperty QPointF QtQuick::PinchEvent::startCenter
-    \qmlproperty QPointF QtQuick::PinchEvent::previousCenter
+    \qmlproperty point QtQuick::PinchEvent::center
+    \qmlproperty point QtQuick::PinchEvent::startCenter
+    \qmlproperty point QtQuick::PinchEvent::previousCenter
 
     These properties hold the position of the center point between the two touch points.
 
@@ -87,10 +87,10 @@ Q_LOGGING_CATEGORY(lcPA, "qt.quick.pincharea")
 */
 
 /*!
-    \qmlproperty QPointF QtQuick::PinchEvent::point1
-    \qmlproperty QPointF QtQuick::PinchEvent::startPoint1
-    \qmlproperty QPointF QtQuick::PinchEvent::point2
-    \qmlproperty QPointF QtQuick::PinchEvent::startPoint2
+    \qmlproperty point QtQuick::PinchEvent::point1
+    \qmlproperty point QtQuick::PinchEvent::startPoint1
+    \qmlproperty point QtQuick::PinchEvent::point2
+    \qmlproperty point QtQuick::PinchEvent::startPoint2
 
     These properties provide the actual touch points generating the pinch.
 
@@ -131,7 +131,7 @@ QQuickPinchAreaPrivate::~QQuickPinchAreaPrivate()
 
 /*!
     \qmltype PinchArea
-    \instantiates QQuickPinchArea
+    \nativetype QQuickPinchArea
     \inqmlmodule QtQuick
     \ingroup qtquick-input
     \inherits Item
@@ -247,7 +247,7 @@ QQuickPinchArea::QQuickPinchArea(QQuickItem *parent)
     Q_D(QQuickPinchArea);
     d->init();
     setAcceptTouchEvents(true);
-#ifdef Q_OS_OSX
+#ifdef Q_OS_MACOS
     setAcceptHoverEvents(true); // needed to enable touch events on mouse hover.
 #endif
 }
