@@ -59,7 +59,7 @@ public:
 
 /*!
    \qmltype StateGroup
-    \instantiates QQuickStateGroup
+    \nativetype QQuickStateGroup
     \inqmlmodule QtQuick
    \ingroup qtquick-states
    \brief Provides built-in state support for non-Item types.
@@ -85,7 +85,7 @@ public:
    }
    \endqml
 
-   \sa {Qt Quick States}{Qt Quick States}, {Animation and Transitions in Qt Quick}{Transitions}, {Qt QML}
+   \sa {Qt Quick States}{Qt Quick States}, {Animation and Transitions in Qt Quick}{Transitions}, {Qt Qml}
 */
 
 QQuickStateGroup::QQuickStateGroup(QObject *parent)
@@ -96,8 +96,10 @@ QQuickStateGroup::QQuickStateGroup(QObject *parent)
 QQuickStateGroup::~QQuickStateGroup()
 {
     Q_D(const QQuickStateGroup);
-    for (int i = 0; i < d->states.size(); ++i)
-        d->states.at(i)->setStateGroup(nullptr);
+    for (QQuickState *state : std::as_const(d->states)) {
+        if (state)
+            state->setStateGroup(nullptr);
+    }
     if (d->nullState)
         d->nullState->setStateGroup(nullptr);
 }
@@ -143,11 +145,9 @@ QQmlListProperty<QQuickState> QQuickStateGroup::statesProperty()
 void QQuickStateGroupPrivate::append_state(QQmlListProperty<QQuickState> *list, QQuickState *state)
 {
     QQuickStateGroup *_this = static_cast<QQuickStateGroup *>(list->object);
-    if (state) {
-        _this->d_func()->states.append(state);
+    _this->d_func()->states.append(state);
+    if (state)
         state->setStateGroup(_this);
-    }
-
 }
 
 qsizetype QQuickStateGroupPrivate::count_state(QQmlListProperty<QQuickState> *list)
@@ -164,34 +164,42 @@ QQuickState *QQuickStateGroupPrivate::at_state(QQmlListProperty<QQuickState> *li
 
 void QQuickStateGroupPrivate::clear_states(QQmlListProperty<QQuickState> *list)
 {
-    QQuickStateGroup *_this = static_cast<QQuickStateGroup *>(list->object);
-    _this->d_func()->setCurrentStateInternal(QString(), true);
-    for (qsizetype i = 0; i < _this->d_func()->states.size(); ++i) {
-        _this->d_func()->states.at(i)->setStateGroup(nullptr);
+    QQuickStateGroupPrivate *d = static_cast<QQuickStateGroup *>(list->object)->d_func();
+    d->setCurrentStateInternal(QString(), true);
+    for (QQuickState *state : std::as_const(d->states)) {
+        if (state)
+            state->setStateGroup(nullptr);
     }
-    _this->d_func()->states.clear();
+    d->states.clear();
 }
 
 void QQuickStateGroupPrivate::replace_states(QQmlListProperty<QQuickState> *list, qsizetype index, QQuickState *state)
 {
-    auto *self = qobject_cast<QQuickStateGroup *>(list->object);
+    auto *self = static_cast<QQuickStateGroup *>(list->object);
     auto *d = self->d_func();
     auto *oldState = d->states.at(index);
     if (oldState != state) {
-        oldState->setStateGroup(nullptr);
-        state->setStateGroup(self);
+        if (oldState)
+            oldState->setStateGroup(nullptr);
+
+        if (state)
+            state->setStateGroup(self);
         d->states.replace(index, state);
-        if (d->currentState == oldState->name())
-            d->setCurrentStateInternal(state->name(), true);
+        if (!oldState || d->currentState == oldState->name())
+            d->setCurrentStateInternal(state ? state->name() : QString(), true);
     }
 }
 
 void QQuickStateGroupPrivate::removeLast_states(QQmlListProperty<QQuickState> *list)
 {
-    auto *d = qobject_cast<QQuickStateGroup *>(list->object)->d_func();
-    if (d->currentState == d->states.last()->name())
-        d->setCurrentStateInternal(d->states.size() > 1 ? d->states.first()->name() : QString(), true);
-    d->states.last()->setStateGroup(nullptr);
+    auto *d = static_cast<QQuickStateGroup *>(list->object)->d_func();
+    if (QQuickState *last = d->states.last()) {
+        if (d->currentState == last->name()) {
+            QQuickState *first = d->states.size() > 1 ? d->states.first() : nullptr;
+            d->setCurrentStateInternal(first ? first->name() : QString(), true);
+        }
+        last->setStateGroup(nullptr);
+    }
     d->states.removeLast();
 }
 
@@ -257,14 +265,14 @@ void QQuickStateGroupPrivate::clear_transitions(QQmlListProperty<QQuickTransitio
   This property is often used in scripts to change between states. For
   example:
 
-  \js
+  \qml
   function toggle() {
       if (button.state == 'On')
           button.state = 'Off';
       else
           button.state = 'On';
   }
-  \endjs
+  \endqml
 
   If the state group is in its base state (i.e. no explicit state has been
   set), \c state will be a blank string. Likewise, you can return a
@@ -300,17 +308,18 @@ void QQuickStateGroup::componentComplete()
 
     QVarLengthArray<QString, 4> names;
     names.reserve(d->states.size());
-    for (int ii = 0; ii < d->states.size(); ++ii) {
-        QQuickState *state = d->states.at(ii);
+    for (QQuickState *state : std::as_const(d->states)) {
+        if (!state)
+            continue;
+
         if (!state->isNamed())
             state->setName(QLatin1String("anonymousState") + QString::number(++d->unnamedCount));
 
         QString stateName = state->name();
-        if (names.contains(stateName)) {
+        if (names.contains(stateName))
             qmlWarning(state->parent()) << "Found duplicate state name: " << stateName;
-        } else {
+        else
             names.append(std::move(stateName));
-        }
     }
 
     if (d->updateAutoState()) {
@@ -338,42 +347,39 @@ bool QQuickStateGroupPrivate::updateAutoState()
         return false;
 
     bool revert = false;
-    for (int ii = 0; ii < states.size(); ++ii) {
-        QQuickState *state = states.at(ii);
-        if (state->isWhenKnown()) {
-            if (state->isNamed()) {
-                bool whenValue = state->when();
-                const QQmlProperty whenProp(state, u"when"_s);
-                const auto potentialWhenBinding = QQmlAnyBinding::ofProperty(whenProp);
-                Q_ASSERT(!potentialWhenBinding.isUntypedPropertyBinding());
+    for (QQuickState *state : std::as_const(states)) {
+        if (!state || !state->isWhenKnown() || !state->isNamed())
+            continue;
 
-                // if there is a binding, the value in when might not be up-to-date at this point
-                // so we manually re-evaluate the binding
-                QQmlAbstractBinding *abstractBinding = potentialWhenBinding.asAbstractBinding();
-                if (abstractBinding && abstractBinding->kind() == QQmlAbstractBinding::QmlBinding) {
-                    QQmlBinding *binding = static_cast<QQmlBinding *>(abstractBinding);
-                    if (binding->hasValidContext()) {
-                        QVariant evalResult = binding->evaluate();
-                        if (evalResult.metaType() == QMetaType::fromType<QJSValue>())
-                            whenValue = evalResult.value<QJSValue>().toBool();
-                        else
-                            whenValue = evalResult.toBool();
-                    }
-                }
+        bool whenValue = state->when();
+        const QQmlPropertyIndex whenIndex(state->metaObject()->indexOfProperty("when"));
+        const auto potentialWhenBinding = QQmlAnyBinding::ofProperty(state, whenIndex);
+        Q_ASSERT(!potentialWhenBinding.isUntypedPropertyBinding());
 
-                if (whenValue) {
-                    qCDebug(lcStates) << "Setting auto state due to expression";
-                    if (currentState != state->name()) {
-                        q->setState(state->name());
-                        return true;
-                    } else {
-                        return false;
-                    }
-                } else if (state->name() == currentState) {
-                    revert = true;
-                }
+        // if there is a binding, the value in when might not be up-to-date at this point
+        // so we manually re-evaluate the binding
+        QQmlAbstractBinding *abstractBinding = potentialWhenBinding.asAbstractBinding();
+        if (abstractBinding && abstractBinding->kind() == QQmlAbstractBinding::QmlBinding) {
+            QQmlBinding *binding = static_cast<QQmlBinding *>(abstractBinding);
+            if (binding->hasValidContext()) {
+                const auto boolType = QMetaType::fromType<bool>();
+                const bool isUndefined = !binding->evaluate(&whenValue, boolType);
+                if (isUndefined)
+                    whenValue = false;
             }
         }
+
+        if (whenValue) {
+            qCDebug(lcStates) << "Setting auto state due to expression";
+            if (currentState != state->name()) {
+                q->setState(state->name());
+                return true;
+            } else {
+                return false;
+            }
+        } else if (state->name() == currentState) {
+            revert = true;
+        }
     }
     if (revert) {
         bool rv = !currentState.isEmpty();
@@ -476,9 +482,9 @@ void QQuickStateGroupPrivate::setCurrentStateInternal(const QString &state,
 
     QQuickState *oldState = nullptr;
     if (!currentState.isEmpty()) {
-        for (int ii = 0; ii < states.size(); ++ii) {
-            if (states.at(ii)->name() == currentState) {
-                oldState = states.at(ii);
+        for (QQuickState *state : std::as_const(states)) {
+            if (state && state->name() == currentState) {
+                oldState = state;
                 break;
             }
         }
@@ -488,9 +494,9 @@ void QQuickStateGroupPrivate::setCurrentStateInternal(const QString &state,
     emit q->stateChanged(currentState);
 
     QQuickState *newState = nullptr;
-    for (int ii = 0; ii < states.size(); ++ii) {
-        if (states.at(ii)->name() == currentState) {
-            newState = states.at(ii);
+    for (QQuickState *state : std::as_const(states)) {
+        if (state && state->name() == currentState) {
+            newState = state;
             break;
         }
     }
@@ -512,9 +518,8 @@ void QQuickStateGroupPrivate::setCurrentStateInternal(const QString &state,
 QQuickState *QQuickStateGroup::findState(const QString &name) const
 {
     Q_D(const QQuickStateGroup);
-    for (int i = 0; i < d->states.size(); ++i) {
-        QQuickState *state = d->states.at(i);
-        if (state->name() == name)
+    for (QQuickState *state : std::as_const(d->states)) {
+        if (state && state->name() == name)
             return state;
     }
 
