@@ -3,8 +3,10 @@
 
 /*---
 esid: sec-array.prototype.reduce
-es5id: 15.4.4.21-3-3
-description: Array.prototype.reduce - value of 'length' is a number (value is 0)
+es5id: 15.4.4.21-3-5
+description: >
+    Array.prototype.reduce - value of 'length' is a number (value is
+    -0)
 ---*/
 
 var accessed = false;
@@ -15,9 +17,8 @@ function callbackfn(prevVal, curVal, idx, obj) {
 }
 
 var obj = {
-  0: 1,
-  1: 1,
-  length: 0
+  0: 11,
+  length: -0
 };
 
 assert.sameValue(Array.prototype.reduce.call(obj, callbackfn, 1), 1, 'Array.prototype.reduce.call(obj, callbackfn, 1)');
