@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.12.1.1-g6-6
+es5id: 15.12.1.1-g6-4
 description: >
-    The JSON lexical grammer allows 'r' as a JSONEscapeCharacter after
+    The JSON lexical grammer allows 'f' as a JSONEscapeCharacter after
     '' in a JSONString
 ---*/
 
-assert.sameValue(JSON.parse('"\\r"'), '\r', 'JSON.parse(\'"\\r"\')');
+assert.sameValue(JSON.parse('"\\f"'), '\f', 'JSON.parse(\'"\\f"\')');
