@@ -5,8 +5,10 @@
 info: |
     No matter how control leaves the embedded 'Statement',
     the scope chain is always restored to its former state
-es5id: 12.10_A3.1_T1
-description: Using "with" statement within global context - normal completion
+es5id: 12.10_A3.5_T1
+description: >
+    Using "with" statement within "for-in" statement, leading to
+    normal completion
 flags: [noStrict]
 ---*/
 
@@ -20,9 +22,10 @@ var myObj = {
     valueOf : function(){return 'obj_valueOf';}
 }
 
-
-with(myObj){
-    p1 = 'x1';
+for(var prop in myObj){
+    with(myObj){
+        p1 = 'x1';
+    }
 }
 
 //////////////////////////////////////////////////////////////////////////////
