@@ -36,54 +36,28 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.9.5.10-9.js';
+gTestfile = '15.9.5.1.js';
 
 /**
-   File Name:          15.9.5.10.js
-   ECMA Section:       15.9.5.10
-   Description:        Date.prototype.getDate
-
-   1.Let t be this time value.
-   2.If t is NaN, return NaN.
-   3.Return DateFromTime(LocalTime(t)).
+   File Name:          15.9.5.1.js
+   ECMA Section:       15.9.5.1 Date.prototype.constructor
+   Description:
+   The initial value of Date.prototype.constructor is the built-in Date
+   constructor.
 
    Author:             christine@netscape.com
    Date:               12 november 1997
 */
 
-var SECTION = "15.9.5.10";
+var SECTION = "15.9.5.1";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Date.prototype.getDate()";
+var TITLE   = "Date.prototype.constructor";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-// some daylight savings time cases
-
-var DST_START_1998 = GetDSTStart(TimeFromYear(1998));
-
-addTestCase( DST_START_1998-1 );
-
 new TestCase( SECTION,
-	      "(new Date(NaN)).getDate()",
-	      NaN,
-	      (new Date(NaN)).getDate() );
-
-new TestCase( SECTION,
-	      "Date.prototype.getDate.length",
-	      0,
-	      Date.prototype.getDate.length );
+	      "Date.prototype.constructor == Date",
+	      true,
+	      Date.prototype.constructor == Date );
 test();
-
-function addTestCase( t ) {
-  var start = TimeFromYear(YearFromTime(t));
-  var stop  = TimeFromYear(YearFromTime(t) + 1);
-
-  for (var d = start; d < stop; d += msPerDay)
-  {
-    new TestCase( SECTION,
-                  "(new Date("+d+")).getDate()",
-                  DateFromTime(LocalTime(d)),
-                  (new Date(d)).getDate() );
-  }
-}
