@@ -1,33 +1,9 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <qtest.h>
 
+#include <QJsonDocument>
 #include <QQmlComponent>
 #include <QQmlEngine>
 #include <QProcess>
@@ -36,6 +12,7 @@
 #include <QSysInfo>
 #include <QLoggingCategory>
 #include <private/qqmlcomponent_p.h>
+#include <private/qqmljscompilerstats_p.h>
 #include <private/qqmlscriptdata_p.h>
 #include <private/qv4compileddata_p.h>
 #include <qtranslator.h>
@@ -45,6 +22,8 @@
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 #include "scriptstringprops.h"
 
+using namespace Qt::StringLiterals;
+
 class tst_qmlcachegen: public QQmlDataTest
 {
     Q_OBJECT
@@ -86,8 +65,14 @@ private slots:
     void inlineComponent();
     void posthocRequired();
 
+    void gracefullyHandleTruncatedCacheFile();
+
     void scriptStringCachegenInteraction();
     void saveableUnitPointer();
+
+    void aotstatsSerialization();
+    void aotstatsGeneration_data();
+    void aotstatsGeneration();
 };
 
 // A wrapper around QQmlComponent to ensure the temporary reference counts
@@ -143,7 +128,9 @@ tst_qmlcachegen::tst_qmlcachegen()
 
 void tst_qmlcachegen::initTestCase()
 {
-    qputenv("QML_FORCE_DISK_CACHE", "1");
+    if (qEnvironmentVariableIsEmpty("QML_DISK_CACHE"))
+        qputenv("QML_FORCE_DISK_CACHE", "1");
+
     QStandardPaths::setTestModeEnabled(true);
 
     // make sure there's no pre-existing cache dir
@@ -154,29 +141,54 @@ void tst_qmlcachegen::initTestCase()
     QQmlDataTest::initTestCase();
 }
 
+#if QT_CONFIG(process)
+static void testWithEnvironment(
+        const QString &function, const QString &testFilePath, const QByteArray &value, bool success)
+{
+    QProcess child;
+    child.setProgram(QCoreApplication::applicationFilePath());
+    child.setArguments(QStringList(function));
+    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
+    env.remove(QLatin1String("QML_FORCE_DISK_CACHE"));
+    env.insert(QLatin1String("QMLCACHEGEN_TEST_FILE_PATH"), testFilePath);
+    env.insert(QLatin1String("QML_DISK_CACHE"), QLatin1String(value));
+    child.setProcessEnvironment(env);
+    child.start();
+    QVERIFY(child.waitForFinished());
+    if (success)
+        QCOMPARE(child.exitCode(), 0);
+    else
+        QVERIFY(child.exitCode() != 0);
+}
+#endif
+
 void tst_qmlcachegen::loadGeneratedFile()
 {
 #if defined(QTEST_CROSS_COMPILED)
     QSKIP("Cannot call qmlcachegen on cross-compiled target.");
 #endif
-
     QTemporaryDir tempDir;
     QVERIFY(tempDir.isValid());
 
-    const auto writeTempFile = [&tempDir](const QString &fileName, const char *contents) {
-        QFile f(tempDir.path() + '/' + fileName);
-        const bool ok = f.open(QIODevice::WriteOnly | QIODevice::Truncate);
-        Q_ASSERT(ok);
-        f.write(contents);
-        return f.fileName();
-    };
+    const QByteArray path = qgetenv("QMLCACHEGEN_TEST_FILE_PATH");
+    QString testFilePath;
+    if (path.isEmpty()) {
+        const auto writeTempFile = [&tempDir](const QString &fileName, const char *contents) {
+            QFile f(tempDir.path() + '/' + fileName);
+            const bool ok = f.open(QIODevice::WriteOnly | QIODevice::Truncate);
+            Q_ASSERT(ok);
+            f.write(contents);
+            return f.fileName();
+        };
 
-    const QString testFilePath = writeTempFile("test.qml", "import QtQml 2.0\n"
-                                                           "QtObject {\n"
-                                                           "    property int value: Math.min(100, 42);\n"
-                                                           "}");
-
-    QVERIFY(generateCache(testFilePath));
+        testFilePath = writeTempFile("test.qml", "import QtQml 2.0\n"
+                                                 "QtObject {\n"
+                                                 "    property int value: Math.min(100, 42);\n"
+                                                 "}");
+        QVERIFY(generateCache(testFilePath));
+    } else {
+        testFilePath = QString::fromUtf8(path);
+    }
 
     const QString cacheFilePath = testFilePath + QLatin1Char('c');
     QVERIFY(QFile::exists(cacheFilePath));
@@ -191,10 +203,21 @@ void tst_qmlcachegen::loadGeneratedFile()
         QCOMPARE(uint(cacheUnit->sourceFileIndex), uint(0));
     }
 
-    QVERIFY(QFile::remove(testFilePath));
+    if (path.isEmpty()) {
+        QVERIFY(QFile::remove(testFilePath));
+    } else {
+#if QT_CONFIG(process)
+        testWithEnvironment("loadGeneratedFile", testFilePath, "qmlc-read", true);
+        testWithEnvironment("loadGeneratedFile", testFilePath, "qmlc-write", false);
+        testWithEnvironment("loadGeneratedFile", testFilePath, "qmlc-read,qmlc-read", true);
+        testWithEnvironment("loadGeneratedFile", testFilePath, "qmlc", true);
+        testWithEnvironment("loadGeneratedFile", testFilePath, "wrong", false);
+#endif
+    }
 
     QQmlEngine engine;
     CleanlyLoadingComponent component(&engine, QUrl::fromLocalFile(testFilePath));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
     QCOMPARE(obj->property("value").toInt(), 42);
@@ -263,6 +286,7 @@ void tst_qmlcachegen::translationExpressionSupport()
 
     QQmlEngine engine;
     CleanlyLoadingComponent component(&engine, QUrl::fromLocalFile(testFilePath));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
     QCOMPARE(obj->property("text").toString(), QString("ALL Ok"));
@@ -309,6 +333,7 @@ void tst_qmlcachegen::signalHandlerParameters()
 
     QQmlEngine engine;
     CleanlyLoadingComponent component(&engine, QUrl::fromLocalFile(testFilePath));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
     QMetaObject::invokeMethod(obj.data(), "runTest");
@@ -332,7 +357,7 @@ void tst_qmlcachegen::signalHandlerParameters()
         };
 
         QVERIFY(isStringIndexInStringTable(compilationUnit->objectAt(0)->signalAt(0)->parameterAt(0)->nameIndex));
-        QVERIFY(!compilationUnit->dynamicStrings.isEmpty());
+        QVERIFY(!compilationUnit->baseCompilationUnit()->dynamicStrings.isEmpty());
     }
 }
 
@@ -395,6 +420,7 @@ void tst_qmlcachegen::aheadOfTimeCompilation()
 
     QQmlEngine engine;
     CleanlyLoadingComponent component(&engine, QUrl::fromLocalFile(testFilePath));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
     QVariant result;
@@ -404,6 +430,22 @@ void tst_qmlcachegen::aheadOfTimeCompilation()
 
 static QQmlPrivate::CachedQmlUnit *temporaryModifiedCachedUnit = nullptr;
 
+static const char *versionCheckErrorString(QQmlMetaType::CachedUnitLookupError error)
+{
+    switch (error) {
+    case QQmlMetaType::CachedUnitLookupError::NoError:
+        return "no error";
+    case QQmlMetaType::CachedUnitLookupError::NoUnitFound:
+        return "no unit found";
+    case QQmlMetaType::CachedUnitLookupError::VersionMismatch:
+        return "version mismatch";
+    case QQmlMetaType::CachedUnitLookupError::NotFullyTyped:
+        return "unit not fully typed";
+    }
+
+    return "wat?";
+}
+
 void tst_qmlcachegen::versionChecksForAheadOfTimeUnits()
 {
     QVERIFY(QFile::exists(":/data/versionchecks.qml"));
@@ -413,9 +455,9 @@ void tst_qmlcachegen::versionChecksForAheadOfTimeUnits()
     QQmlMetaType::CachedUnitLookupError error = QQmlMetaType::CachedUnitLookupError::NoError;
     QLoggingCategory::setFilterRules("qt.qml.diskcache.debug=true");
     const QQmlPrivate::CachedQmlUnit *originalUnit = QQmlMetaType::findCachedCompilationUnit(
-            QUrl("qrc:/data/versionchecks.qml"), &error);
+            QUrl("qrc:/data/versionchecks.qml"), QQmlMetaType::AcceptUntyped, &error);
     QLoggingCategory::setFilterRules(QString());
-    QVERIFY(originalUnit);
+    QVERIFY2(originalUnit, versionCheckErrorString(error));
     QV4::CompiledData::Unit *tweakedUnit = (QV4::CompiledData::Unit *)malloc(originalUnit->qmlData->unitSize);
     memcpy(reinterpret_cast<void *>(tweakedUnit),
            reinterpret_cast<const void *>(originalUnit->qmlData),
@@ -442,7 +484,8 @@ void tst_qmlcachegen::versionChecksForAheadOfTimeUnits()
 
     {
         QQmlMetaType::CachedUnitLookupError error = QQmlMetaType::CachedUnitLookupError::NoError;
-        QVERIFY(!QQmlMetaType::findCachedCompilationUnit(QUrl("qrc:/data/versionchecks.qml"), &error));
+        QVERIFY(!QQmlMetaType::findCachedCompilationUnit(
+                    QUrl("qrc:/data/versionchecks.qml"), QQmlMetaType::AcceptUntyped, &error));
         QCOMPARE(error, QQmlMetaType::CachedUnitLookupError::VersionMismatch);
     }
 
@@ -468,7 +511,7 @@ void tst_qmlcachegen::skippedResources()
 
     QQmlMetaType::CachedUnitLookupError error = QQmlMetaType::CachedUnitLookupError::NoError;
     const QQmlPrivate::CachedQmlUnit *unit = QQmlMetaType::findCachedCompilationUnit(
-            QUrl("qrc:/not/Skip.qml"), &error);
+            QUrl("qrc:/not/Skip.qml"), QQmlMetaType::AcceptUntyped, &error);
     QCOMPARE(unit, nullptr);
     QCOMPARE(error, QQmlMetaType::CachedUnitLookupError::NoUnitFound);
 }
@@ -539,6 +582,7 @@ void tst_qmlcachegen::functionExpressions()
 
     QQmlEngine engine;
     CleanlyLoadingComponent component(&engine, QUrl::fromLocalFile(testFilePath));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
 
@@ -585,6 +629,23 @@ void tst_qmlcachegen::qrcScriptImport()
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
     QTRY_COMPARE(obj->property("value").toInt(), 42);
+#if QT_CONFIG(process)
+    if (qEnvironmentVariableIsEmpty("QMLCACHEGEN_TEST_FILE_PATH")) {
+        testWithEnvironment("qrcScriptImport", ":/data/jsimport.qml", "aot-native", false);
+        testWithEnvironment("qrcScriptImport", ":/data/jsimport.qml", "aot-bytecode", true);
+        testWithEnvironment("qrcScriptImport", ":/data/jsimport.qml", "aot-bytecode,aot-native", true);
+        testWithEnvironment("qrcScriptImport", ":/data/jsimport.qml", "aot", true);
+        testWithEnvironment("qrcScriptImport", ":/data/jsimport.qml", "wrong", false);
+    }
+#endif
+
+    auto componentPrivate = QQmlComponentPrivate::get(&component);
+    QVERIFY(componentPrivate);
+    auto compilationUnit = componentPrivate->compilationUnit;
+    QVERIFY(compilationUnit);
+    auto unitData = compilationUnit->unitData();
+    QVERIFY(unitData);
+    QVERIFY(unitData->flags & QV4::CompiledData::Unit::StaticData);
 }
 
 void tst_qmlcachegen::fsScriptImport()
@@ -640,6 +701,7 @@ void tst_qmlcachegen::fsScriptImport()
 
     QQmlEngine engine;
     CleanlyLoadingComponent component(&engine, QUrl::fromLocalFile(testFilePath));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
     QCOMPARE(obj->property("value").toInt(), 42);
@@ -660,7 +722,8 @@ void tst_qmlcachegen::moduleScriptImport()
     {
         auto componentPrivate = QQmlComponentPrivate::get(&component);
         QVERIFY(componentPrivate);
-        auto compilationUnit = componentPrivate->compilationUnit->dependentScripts.first()->compilationUnit();
+        auto compilationUnit = componentPrivate->compilationUnit->dependentScriptsPtr()
+                                       ->first()->compilationUnit();
         QVERIFY(compilationUnit);
         auto unitData = compilationUnit->unitData();
         QVERIFY(unitData);
@@ -669,7 +732,7 @@ void tst_qmlcachegen::moduleScriptImport()
 
         QQmlMetaType::CachedUnitLookupError error = QQmlMetaType::CachedUnitLookupError::NoError;
         const QQmlPrivate::CachedQmlUnit *unitFromResources = QQmlMetaType::findCachedCompilationUnit(
-                QUrl("qrc:/data/script.mjs"), &error);
+                QUrl("qrc:/data/script.mjs"), QQmlMetaType::AcceptUntyped, &error);
         QVERIFY(unitFromResources);
 
         QCOMPARE(unitFromResources->qmlData, compilationUnit->unitData());
@@ -700,7 +763,7 @@ void tst_qmlcachegen::sourceFileIndices()
 
     QQmlMetaType::CachedUnitLookupError error = QQmlMetaType::CachedUnitLookupError::NoError;
     const QQmlPrivate::CachedQmlUnit *unitFromResources = QQmlMetaType::findCachedCompilationUnit(
-            QUrl("qrc:/data/versionchecks.qml"), &error);
+            QUrl("qrc:/data/versionchecks.qml"), QQmlMetaType::AcceptUntyped, &error);
     QVERIFY(unitFromResources);
     QVERIFY(unitFromResources->qmlData->flags & QV4::CompiledData::Unit::PendingTypeCompilation);
     QCOMPARE(uint(unitFromResources->qmlData->sourceFileIndex), uint(0));
@@ -762,6 +825,7 @@ void tst_qmlcachegen::inlineComponent()
     QVERIFY2(ok, errors);
     QQmlEngine engine;
     CleanlyLoadingComponent component(&engine, testFileUrl("inlineComponentWithId.qml"));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
     QTest::ignoreMessage(QtMsgType::QtInfoMsg, "42");
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(!obj.isNull());
@@ -779,7 +843,26 @@ void tst_qmlcachegen::posthocRequired()
     CleanlyLoadingComponent component(&engine, testFileUrl("posthocrequired.qml"));
     QScopedPointer<QObject> obj(component.create());
     QVERIFY(obj.isNull() && component.isError());
-    QVERIFY(component.errorString().contains(QStringLiteral("Required property x was not initialized")));
+    QVERIFY2(component.errorString().contains(
+                 QStringLiteral("Required property x was not initialized")),
+             qPrintable(component.errorString()));
+}
+
+void tst_qmlcachegen::gracefullyHandleTruncatedCacheFile()
+{
+#if defined(QTEST_CROSS_COMPILED)
+    QSKIP("Cannot call qmlcachegen on cross-compiled target.");
+#endif
+
+    bool ok = generateCache(testFile("truncateTest.qml"));
+    QVERIFY(ok);
+    const QString qmlcFile = testFile("truncateTest.qmlc");
+    QVERIFY(QFile::exists(qmlcFile));
+    QFile::resize(qmlcFile, QFileInfo(qmlcFile).size() / 2);
+    QQmlEngine engine;
+    CleanlyLoadingComponent component(&engine, testFileUrl("truncateTest.qml"));
+    QScopedPointer<QObject> obj(component.create());
+    QVERIFY(!obj.isNull());
 }
 
 void tst_qmlcachegen::scriptStringCachegenInteraction()
@@ -799,7 +882,7 @@ void tst_qmlcachegen::scriptStringCachegenInteraction()
 
     QVERIFY(scripty->m_undef.isUndefinedLiteral());
     QVERIFY(scripty->m_nul.isNullLiteral());
-    QCOMPARE(scripty->m_str.stringLiteral(), u"hello"_qs);
+    QCOMPARE(scripty->m_str.stringLiteral(), u"hello"_s);
     QCOMPARE(scripty->m_num.numberLiteral(&ok), 42);
     ok = false;
     scripty->m_bol.booleanLiteral(&ok);
@@ -818,6 +901,134 @@ void tst_qmlcachegen::saveableUnitPointer()
     QCOMPARE(unit.flags, flags);
 }
 
+void tst_qmlcachegen::aotstatsSerialization()
+{
+    const auto createEntry = [](const auto &d, const auto &n, const auto &e, const auto &l,
+                                const auto &c, const auto &s) -> QQmlJS::AotStatsEntry {
+        QQmlJS::AotStatsEntry entry;
+        entry.codegenDuration = d;
+        entry.functionName = n;
+        entry.errorMessage = e;
+        entry.line = l;
+        entry.column = c;
+        entry.codegenSuccessful = s;
+        return entry;
+    };
+
+    const auto equal = [](const auto &e1, const auto &e2) -> bool {
+        return e1.codegenDuration == e2.codegenDuration && e1.functionName == e2.functionName
+                && e1.errorMessage == e2.errorMessage && e1.line == e2.line
+                && e1.column == e2.column && e1.codegenSuccessful == e2.codegenSuccessful;
+    };
+
+    // AotStats
+    // +-ModuleA
+    // | +-File1
+    // | | +-e1
+    // | | +-e2
+    // | +-File2
+    // | | +-e3
+    // +-ModuleB
+    // | +-File3
+    // | | +-e4
+
+    QQmlJS::AotStats original;
+    QQmlJS::AotStatsEntry e1 = createEntry(std::chrono::microseconds(500), "f1", "", 1, 1, true);
+    QQmlJS::AotStatsEntry e2 = createEntry(std::chrono::microseconds(200), "f2", "err1", 5, 4, false);
+    QQmlJS::AotStatsEntry e3 = createEntry(std::chrono::microseconds(750), "f3", "", 20, 4, true);
+    QQmlJS::AotStatsEntry e4 = createEntry(std::chrono::microseconds(300), "f4", "err2", 5, 8, false);
+    original.addEntry("ModuleA", "File1", e1);
+    original.addEntry("ModuleA", "File1", e2);
+    original.addEntry("ModuleA", "File2", e3);
+    original.addEntry("ModuleB", "File3", e4);
+
+    const auto parsed = QQmlJS::AotStats::fromJsonDocument(original.toJsonDocument());
+    QCOMPARE(parsed.entries().size(), original.entries().size());
+
+    const auto &parsedA = parsed.entries()["ModuleA"];
+    const auto &originalA = original.entries()["ModuleA"];
+    QCOMPARE(parsedA.size(), originalA.size());
+    QCOMPARE(parsedA["File1"].size(), originalA["File1"].size());
+    QVERIFY(equal(parsedA["File1"][0], originalA["File1"][0]));
+    QVERIFY(equal(parsedA["File1"][1], originalA["File1"][1]));
+    QCOMPARE(parsedA["File2"].size(), originalA["File2"].size());
+    QVERIFY(equal(parsedA["File2"][0], originalA["File2"][0]));
+
+    const auto &parsedB = parsed.entries()["ModuleB"];
+    const auto &originalB = original.entries()["ModuleB"];
+    QCOMPARE(parsedB.size(), originalB.size());
+    QCOMPARE(parsedB["File3"].size(), originalB["File3"].size());
+    QVERIFY(equal(parsedB["File3"][0], originalB["File3"][0]));
+}
+
+struct FunctionEntry
+{
+    QString name;
+    QString errorMessage;
+    bool codegenSuccessful;
+};
+
+void tst_qmlcachegen::aotstatsGeneration_data()
+{
+    QTest::addColumn<QString>("qmlFile");
+    QTest::addColumn<QList<FunctionEntry>>("entries");
+
+    QTest::addRow("clean") << "AotstatsClean.qml"
+                           << QList<FunctionEntry>{ { "j", "", true }, { "s", "", true } };
+
+    const QString fError = "function without return type annotation returns int. This may prevent "
+                           "proper compilation to Cpp.";
+    const QString sError = "method g cannot be resolved.";
+    QTest::addRow("mixed") << "AotstatsMixed.qml"
+                           << QList<FunctionEntry>{ { "i", "", true },
+                                                    { "f", fError, false },
+                                                    { "s", sError, false } };
+}
+
+void tst_qmlcachegen::aotstatsGeneration()
+{
+#if defined(QTEST_CROSS_COMPILED)
+    QSKIP("Cannot call qmlcachegen on cross-compiled target.");
+#endif
+    QFETCH(QString, qmlFile);
+    QFETCH(QList<FunctionEntry>, entries);
+
+    QTemporaryDir dir;
+    QProcess proc;
+    proc.setProgram(QLibraryInfo::path(QLibraryInfo::LibraryExecutablesPath) + "/qmlcachegen"_L1);
+    const QString cppOutput = dir.filePath(qmlFile + ".cpp");
+    const QString aotstatsOutput = cppOutput + ".aotstats";
+    proc.setArguments({ "--bare",
+                        "--resource-path", "/cachegentest/data/aotstats/" + qmlFile,
+                        "-i", testFile("aotstats/qmldir"),
+                        "--resource", testFile("aotstats/cachegentest.qrc"),
+                        "--dump-aot-stats",
+                        "--module-id=Aotstats",
+                        "-o", cppOutput,
+                        testFile("aotstats/" + qmlFile) });
+    proc.start();
+    QVERIFY(proc.waitForFinished() && proc.exitStatus() == QProcess::NormalExit);
+
+    QVERIFY(QFileInfo::exists(aotstatsOutput));
+    QFile aotstatsFile(aotstatsOutput);
+    QVERIFY(aotstatsFile.open(QIODevice::Text | QIODevice::ReadOnly));
+    const auto document = QJsonDocument::fromJson(aotstatsFile.readAll());
+    const auto aotstats = QQmlJS::AotStats::fromJsonDocument(document);
+    QVERIFY(aotstats.entries().size() == 1);  // One module
+    const auto &moduleEntries = aotstats.entries()["Aotstats"];
+    QVERIFY(moduleEntries.size() == 1);     // Only one qml file was compiled
+    QCOMPARE(moduleEntries.keys().first(), testFile("aotstats/" + qmlFile));
+    const auto &fileEntries = moduleEntries[moduleEntries.keys().first()];
+
+    for (const auto &entry : entries) {
+        const auto it = std::find_if(fileEntries.cbegin(), fileEntries.cend(),
+                                     [&](const auto &e) { return e.functionName == entry.name; });
+        QVERIFY(it != fileEntries.cend());
+        QVERIFY(it->codegenSuccessful == entry.codegenSuccessful);
+        QVERIFY(it->errorMessage == entry.errorMessage);
+    }
+}
+
 const QQmlScriptString &ScriptStringProps::undef() const
 {
     return m_undef;
