@@ -1,5 +1,5 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef FOO_H
 #define FOO_H
@@ -12,6 +12,12 @@ class Bbb : public QObject
     Q_OBJECT
 public:
     Bbb(QObject *parent = nullptr) : QObject(parent) {}
+
+Q_SIGNALS:
+    void mySignal(QObject *myObject, const QObject *myConstObject, QObject const *myConstObject2,
+                  QObject *const myObject2, const QObject *const myConstObject3);
+    void myVolatileSignal(volatile const QObject *a, const volatile QObject *b,
+                          volatile QObject *nonConst);
 };
 
 class Ccc : public QObject
