@@ -32,6 +32,8 @@ private slots:
     void findFilePathsFromFileNames_data();
     void findFilePathsFromFileNames();
     void openFiles();
+    void reloadLotsOfFiles();
+    void importPathViaSettings();
 };
 
 #endif // TST_QMLLS_QQMLCODEMODEL_H
