@@ -1,31 +1,7 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
+#include <QtCore/private/qvariantanimation_p.h>
 #include <QtTest/QtTest>
 #include <QtTest/QSignalSpy>
 #include <QtGui/QStyleHints>
@@ -33,11 +9,15 @@
 #include <QtGui/private/qeventpoint_p.h>
 #include <qpa/qwindowsysteminterface.h>
 #include <private/qquickpincharea_p.h>
+#include <QtQuick/private/qquickpathview_p.h>
 #include <QtQuick/private/qquickrectangle_p.h>
 #include <QtQuick/qquickview.h>
 #include <QtQml/qqmlcontext.h>
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 #include <QtQuickTestUtils/private/viewtestutils_p.h>
+#include <QtQuickTestUtils/private/visualtestutils_p.h>
+
+using namespace QQuickVisualTestUtils;
 
 class tst_QQuickPinchArea: public QQmlDataTest
 {
@@ -55,6 +35,8 @@ private slots:
     void transformedPinchArea();
     void dragTransformedPinchArea_data();
     void dragTransformedPinchArea();
+    void pinchAreaKeepsDragInView();
+    void pinchInPathView();
 
 private:
     QQuickView *createView();
@@ -85,18 +67,18 @@ void tst_QQuickPinchArea::pinchProperties()
     QVERIFY(rootItem != nullptr);
     QSignalSpy targetSpy(pinch, SIGNAL(targetChanged()));
     pinch->setTarget(rootItem);
-    QCOMPARE(targetSpy.count(),1);
+    QCOMPARE(targetSpy.size(),1);
     pinch->setTarget(rootItem);
-    QCOMPARE(targetSpy.count(),1);
+    QCOMPARE(targetSpy.size(),1);
 
     // axis
     QCOMPARE(pinch->axis(), QQuickPinch::XAndYAxis);
     QSignalSpy axisSpy(pinch, SIGNAL(dragAxisChanged()));
     pinch->setAxis(QQuickPinch::XAxis);
     QCOMPARE(pinch->axis(), QQuickPinch::XAxis);
-    QCOMPARE(axisSpy.count(),1);
+    QCOMPARE(axisSpy.size(),1);
     pinch->setAxis(QQuickPinch::XAxis);
-    QCOMPARE(axisSpy.count(),1);
+    QCOMPARE(axisSpy.size(),1);
 
     // minimum and maximum drag properties
     QSignalSpy xminSpy(pinch, SIGNAL(minimumXChanged()));
@@ -119,20 +101,20 @@ void tst_QQuickPinchArea::pinchProperties()
     QCOMPARE(pinch->ymin(), 10.0);
     QCOMPARE(pinch->ymax(), 10.0);
 
-    QCOMPARE(xminSpy.count(),1);
-    QCOMPARE(xmaxSpy.count(),1);
-    QCOMPARE(yminSpy.count(),1);
-    QCOMPARE(ymaxSpy.count(),1);
+    QCOMPARE(xminSpy.size(),1);
+    QCOMPARE(xmaxSpy.size(),1);
+    QCOMPARE(yminSpy.size(),1);
+    QCOMPARE(ymaxSpy.size(),1);
 
     pinch->setXmin(10);
     pinch->setXmax(10);
     pinch->setYmin(10);
     pinch->setYmax(10);
 
-    QCOMPARE(xminSpy.count(),1);
-    QCOMPARE(xmaxSpy.count(),1);
-    QCOMPARE(yminSpy.count(),1);
-    QCOMPARE(ymaxSpy.count(),1);
+    QCOMPARE(xminSpy.size(),1);
+    QCOMPARE(xmaxSpy.size(),1);
+    QCOMPARE(yminSpy.size(),1);
+    QCOMPARE(ymaxSpy.size(),1);
 
     // minimum and maximum scale properties
     QSignalSpy scaleMinSpy(pinch, SIGNAL(minimumScaleChanged()));
@@ -147,14 +129,14 @@ void tst_QQuickPinchArea::pinchProperties()
     QCOMPARE(pinch->minimumScale(), 0.5);
     QCOMPARE(pinch->maximumScale(), 1.5);
 
-    QCOMPARE(scaleMinSpy.count(),1);
-    QCOMPARE(scaleMaxSpy.count(),1);
+    QCOMPARE(scaleMinSpy.size(),1);
+    QCOMPARE(scaleMaxSpy.size(),1);
 
     pinch->setMinimumScale(0.5);
     pinch->setMaximumScale(1.5);
 
-    QCOMPARE(scaleMinSpy.count(),1);
-    QCOMPARE(scaleMaxSpy.count(),1);
+    QCOMPARE(scaleMinSpy.size(),1);
+    QCOMPARE(scaleMaxSpy.size(),1);
 
     // minimum and maximum rotation properties
     QSignalSpy rotMinSpy(pinch, SIGNAL(minimumRotationChanged()));
@@ -169,22 +151,22 @@ void tst_QQuickPinchArea::pinchProperties()
     QCOMPARE(pinch->minimumRotation(), -90.0);
     QCOMPARE(pinch->maximumRotation(), 45.0);
 
-    QCOMPARE(rotMinSpy.count(),1);
-    QCOMPARE(rotMaxSpy.count(),1);
+    QCOMPARE(rotMinSpy.size(),1);
+    QCOMPARE(rotMaxSpy.size(),1);
 
     pinch->setMinimumRotation(-90.0);
     pinch->setMaximumRotation(45.0);
 
-    QCOMPARE(rotMinSpy.count(),1);
-    QCOMPARE(rotMaxSpy.count(),1);
+    QCOMPARE(rotMinSpy.size(),1);
+    QCOMPARE(rotMaxSpy.size(),1);
 }
 
-QMutableEventPoint makeTouchPoint(int id, QPoint p, QQuickView *v, QQuickItem *i)
+QEventPoint makeTouchPoint(int id, QPoint p, QQuickView *v, QQuickItem *i)
 {
-    QMutableEventPoint touchPoint(id);
-    touchPoint.setPosition(i->mapFromScene(p));
-    touchPoint.setGlobalPosition(v->mapToGlobal(p));
-    touchPoint.setScenePosition(p);
+    QEventPoint touchPoint(id);
+    QMutableEventPoint::setPosition(touchPoint, i->mapFromScene(p));
+    QMutableEventPoint::setGlobalPosition(touchPoint, v->mapToGlobal(p));
+    QMutableEventPoint::setScenePosition(touchPoint, p);
     return touchPoint;
 }
 
@@ -409,7 +391,7 @@ void tst_QQuickPinchArea::retouch()
         pinchSequence.move(0, p1,window).move(1, p2,window).commit();
         QQuickTouchUtils::flush(window);
 
-        QCOMPARE(startedSpy.count(), 1);
+        QCOMPARE(startedSpy.size(), 1);
 
         QCOMPARE(root->property("scale").toReal(), 1.5);
         QCOMPARE(root->property("center").toPointF(), QPointF(40, 40)); // blackrect is at 50,50
@@ -417,15 +399,15 @@ void tst_QQuickPinchArea::retouch()
 
         QCOMPARE(window->rootObject()->property("pointCount").toInt(), 2);
 
-        QCOMPARE(startedSpy.count(), 1);
-        QCOMPARE(finishedSpy.count(), 0);
+        QCOMPARE(startedSpy.size(), 1);
+        QCOMPARE(finishedSpy.size(), 0);
 
         // Hold down the first finger but release the second one
         pinchSequence.stationary(0).release(1, p2, window).commit();
         QQuickTouchUtils::flush(window);
 
-        QCOMPARE(startedSpy.count(), 1);
-        QCOMPARE(finishedSpy.count(), 0);
+        QCOMPARE(startedSpy.size(), 1);
+        QCOMPARE(finishedSpy.size(), 0);
 
         QCOMPARE(window->rootObject()->property("pointCount").toInt(), 1);
 
@@ -438,8 +420,8 @@ void tst_QQuickPinchArea::retouch()
         QQuickTouchUtils::flush(window);
 
         // Lifting and retouching results in onPinchStarted being called again
-        QCOMPARE(startedSpy.count(), 2);
-        QCOMPARE(finishedSpy.count(), 0);
+        QCOMPARE(startedSpy.size(), 2);
+        QCOMPARE(finishedSpy.size(), 0);
 
         QCOMPARE(window->rootObject()->property("pointCount").toInt(), 2);
 
@@ -447,8 +429,8 @@ void tst_QQuickPinchArea::retouch()
         QQuickTouchUtils::flush(window);
 
         QVERIFY(!root->property("pinchActive").toBool());
-        QCOMPARE(startedSpy.count(), 2);
-        QCOMPARE(finishedSpy.count(), 1);
+        QCOMPARE(startedSpy.size(), 2);
+        QCOMPARE(finishedSpy.size(), 1);
     }
 }
 
@@ -643,6 +625,134 @@ void tst_QQuickPinchArea::dragTransformedPinchArea() // QTBUG-63673
     QCOMPARE(pinchArea->pinch()->active(), false);
 }
 
+// QTBUG-105058
+void tst_QQuickPinchArea::pinchAreaKeepsDragInView()
+{
+    QQuickView view;
+    view.setSource(testFileUrl("pinchAreaInPathView.qml"));
+    QVERIFY(view.rootObject());
+    view.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&view));
+
+    QQuickPathView *pathView = qobject_cast<QQuickPathView*>(view.rootObject());
+    QVERIFY(pathView);
+    QCOMPARE(pathView->count(), 3);
+
+    const QQuickItem *pinchDelegateItem = pathView->itemAtIndex(0);
+    QQuickPinchArea *pinchArea = pinchDelegateItem->property("pinchArea").value<QQuickPinchArea*>();
+    QVERIFY(pinchArea);
+
+    // Press.
+    QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(&view, device);
+    QPoint point1Start = { 80, 120 };
+    QPoint point2Start = { 120, 80 };
+    const int dragThreshold = qApp->styleHints()->startDragDistance();
+    pinchSequence.press(1, pinchArea->mapToScene(point1Start).toPoint(), &view)
+                 .press(2, pinchArea->mapToScene(point2Start).toPoint(), &view).commit();
+    QQuickTouchUtils::flush(&view);
+
+    // Move past the drag threshold to begin the pinch.
+    const int steps = 30;
+    QPoint point1End = point1Start + QPoint(-dragThreshold, dragThreshold);
+    QPoint point2End = point2Start + QPoint(dragThreshold, -dragThreshold);
+
+    forEachStep(steps, [&](qreal progress) {
+        pinchSequence.move(1, lerpPoints(point1Start, point1End, progress), &view)
+                     .move(2, lerpPoints(point2Start, point2End, progress), &view).commit();
+        QQuickTouchUtils::flush(&view);
+        QTest::qWait(5);
+    });
+    QCOMPARE(pinchArea->pinch()->active(), true);
+    QVERIFY2(pinchDelegateItem->scale() > 1.0, qPrintable(QString::number(pinchDelegateItem->scale())));
+    // The PathView contents shouldn't have moved.
+    QCOMPARE(pathView->offset(), 0);
+
+    // Release a touch point.
+    pinchSequence.stationary(1).release(2, point2End, &view).commit();
+    QQuickTouchUtils::flush(&view);
+
+    // Press it again.
+    pinchSequence.stationary(1).press(2, point2End, &view).commit();
+    QQuickTouchUtils::flush(&view);
+    QCOMPARE(pinchArea->pinch()->active(), true);
+
+    // Drag to the right; the PathView still shouldn't move.
+    point1Start = point1End;
+    point2Start = point2End;
+    point1End = point1Start + QPoint(100, 0);
+    point2End = point2Start + QPoint(100, 0);
+    forEachStep(steps, [&](qreal progress) {
+        pinchSequence.move(1, lerpPoints(point1Start, point1End, progress), &view)
+                     .move(2, lerpPoints(point2Start, point2End, progress), &view).commit();
+        QQuickTouchUtils::flush(&view);
+        QTest::qWait(5);
+    });
+    QCOMPARE(pinchArea->pinch()->active(), true);
+    QVERIFY2(pinchDelegateItem->scale() > 1.0, qPrintable(QString::number(pinchDelegateItem->scale())));
+    QCOMPARE(pathView->offset(), 0);
+
+    // Release pinch.
+    pinchSequence.release(1, point1End, &view).release(2, point2End, &view).commit();
+    QQuickTouchUtils::flush(&view);
+    QCOMPARE(pinchArea->pinch()->active(), false);
+    QCOMPARE(pathView->offset(), 0);
+}
+
+void tst_QQuickPinchArea::pinchInPathView()
+{
+    QQuickView view;
+    view.setSource(testFileUrl("pinchAreaInPathView.qml"));
+    QVERIFY(view.rootObject());
+    view.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&view));
+
+    QQuickPathView *pathView = qobject_cast<QQuickPathView*>(view.rootObject());
+    QVERIFY(pathView);
+    QCOMPARE(pathView->count(), 3);
+
+    const QQuickItem *pinchDelegateItem = pathView->itemAtIndex(0);
+    QQuickPinchArea *pinchArea = pinchDelegateItem->property("pinchArea").value<QQuickPinchArea*>();
+    QVERIFY(pinchArea);
+
+    // press
+    QTest::QTouchEventSequence pinchSequence = QTest::touchEvent(&view, device);
+    QPoint point1Start = { 10, 10 };
+    QPoint point2Start = { 100, 100 };
+    const int dragThreshold = qApp->styleHints()->startDragDistance();
+    pinchSequence.press(0, point1Start, &view)
+                 .press(1, point2Start, &view)
+                 .commit();
+    QQuickTouchUtils::flush(&view);
+    QTest::qWait(20);
+
+    // move
+    QPoint moveDistance = QPoint(dragThreshold * 3, dragThreshold * 3);
+    QPoint point2End = point2Start + moveDistance;
+    pinchSequence.stationary(0)
+                 .move(1, point2End, &view)
+                 .commit();
+    QQuickTouchUtils::flush(&view);
+    QTest::qWait(20);
+
+    point2End += moveDistance;
+    pinchSequence.stationary(0)
+                 .move(1, point2End, &view)
+                 .commit();
+    QQuickTouchUtils::flush(&view);
+    QTest::qWait(20);
+
+    QCOMPARE(pinchArea->pinch()->active(), true);
+    QVERIFY2(pinchDelegateItem->scale() > 1.0, qPrintable(QString::number(pinchDelegateItem->scale())));
+    // PathView shouldn't have moved.
+    QCOMPARE(pathView->offset(), 0);
+
+    // release pinch.
+    pinchSequence.release(0, point1Start, &view).release(1, point2End, &view).commit();
+    QQuickTouchUtils::flush(&view);
+    QCOMPARE(pinchArea->pinch()->active(), false);
+    QCOMPARE(pathView->offset(), 0);
+}
+
 QQuickView *tst_QQuickPinchArea::createView()
 {
     QQuickView *window = new QQuickView(nullptr);
